@@ -991,7 +991,7 @@ function compare_symbol(sym1, sym2) {
 //======================================================================
 // start Pspm object
 //======================================================================
-var Pspm = function(matrix, name, ltrim, rtrim, nsites, evalue, pssm, alt) {
+var Pspm = function(matrix, name, ltrim, rtrim, nsites, evalue, pssm, alt, pgm) {
   "use strict";
   var row, col, data, row_sum, delta, evalue_re;
   if (typeof name !== "string") {
@@ -1010,6 +1010,7 @@ var Pspm = function(matrix, name, ltrim, rtrim, nsites, evalue, pssm, alt) {
     this.ltrim = matrix.ltrim;
     this.rtrim = matrix.rtrim;
     this.pspm = [];
+    this.pgm = matrix.pgm;
     for (row = 0; row < matrix.motif_length; row++) {
       this.pspm[row] = [];
       for (col = 0; col < matrix.alph_length; col++) {
@@ -1065,6 +1066,7 @@ var Pspm = function(matrix, name, ltrim, rtrim, nsites, evalue, pssm, alt) {
     this.evalue = evalue;
     this.ltrim = ltrim;
     this.rtrim = rtrim;
+    this.pgm = pgm;
     if (typeof matrix === "string") {
       // string constructor
       data = parse_pspm_string(matrix);
@@ -1386,7 +1388,8 @@ Pspm.prototype.as_pspm = function() {
   "use strict";
   return "letter-probability matrix: alength= " + this.alph_length + 
       " w= " + this.motif_length + " nsites= " + this.nsites + 
-      " E= " + (typeof this.evalue === "number" ? 
+      (this.pgm === "STREME" ? " P= " : " E= ") + 
+          (typeof this.evalue === "number" ? 
           this.evalue.toExponential() : this.evalue) + "\n" +
       this.as_probability_matrix();
 };
@@ -2703,3 +2706,36 @@ function trim (str) {
   while (ws.test(str.charAt(--i)));
   return str.slice(0, i + 1);
 }
+
+//
+// Delay drawing a logo
+//
+var DelayLogoTask = function(logo, canvas) {
+  "use strict";
+  canvas.width = canvas.width; // clear canvas
+  this.logo = logo;
+  this.canvas = canvas;
+};
+
+DelayLogoTask.prototype.run = function () {
+  "use strict";
+  this.canvas.width = this.canvas.width; // clear canvas
+  draw_logo_on_canvas(this.logo, this.canvas, false);
+};
+
+/*
+ * Make a canvas with the motif logo drawn on it.
+ */
+function make_logo(alphabet, pspm, height, rc, offset, className) {
+  if (rc) pspm = pspm.copy().reverse_complement(alphabet);
+  var logo = new Logo(alphabet);
+  logo.add_pspm(pspm, offset);
+  var canvas = document.createElement('canvas');
+  var sizeit = (height < 0);
+  canvas.height = (sizeit ? -height : height); 
+  canvas.width = 0;
+  canvas.className = className;
+  if (sizeit) size_logo_on_canvas(logo, canvas, false);
+  add_draw_task(canvas, new DelayLogoTask(logo, canvas));
+  return canvas;
+}
