@@ -9,6 +9,30 @@ var shared_doc = {
 
   /* Fairly common options  */
 
+  "all-desc-opt": ["desc"],
+  "all-desc-itm": ["<span class='pdat'>description</span>"],
+  "all-desc-desc": [
+    "Include the text <span class='pdat'>description</span> in the \"Description\" section of the " +
+    "HTML output, taking into account newlines (line breaks) and multiple newlines (paragraph breaks)."
+  ],
+  "all-desc-def": [
+    "No description in the HTML output."
+  ],
+
+  "all-dfile-opt": ["dfile"],
+  "all-dfile-itm": ["<span class='pdat'>dfile</span>"],
+  "all-dfile-desc": [
+    "Include the first 500 characters of text from the file named " + 
+    "<span class='pdat'>dfile</span> in the \"Description\" secton of the HTML output, " +
+    "taking into account newlines (line breaks) and multiple newlines (paragraph breaks). " +
+    "This option overrides option <span class='popt'>",
+    {"if": "ddash", "yes": "-"},
+    "-desc</span>."
+  ],
+  "all-dfile-def": [
+    "No description in the HTML output."
+  ],
+
   "all-help-opt": [{"if": "short", "yes": "h", "no": "help"}],
   "all-help-desc": [
     "Display the usage message and exit."
@@ -99,7 +123,7 @@ var shared_doc = {
     {"if": "spamo", "yes": "<b>secondary</b> "},
     {"if": "centrimo", "yes": ""},
     "motifs with names matching the pattern. The pattern can ",
-    "contain shell-like wildcards (e.g., '*') though they must be escaped ",
+    "contain shell-like wildcards (e.g., \"<code>*</code>\") though they must be escaped ",
     "or quoted to prevent the shell from auto-expanding them. ",
     "This option may be may be repeated, and ",
     {"if": "inc", "yes": "<b>only</b> ", "no": "any "},
@@ -202,8 +226,9 @@ var shared_doc = {
         }
       ]
     },
-    "Incompatible with options <span class=\"popt\">-dna</span>, <span class=\"popt\">-rna</span> ",
-    "and <span class=\"popt\">-protein</span>. "
+    "Incompatible with options <span class=\"popt\">", {"if": "ddash", "yes": ["-"]}, "-dna</span>, <span class=\"popt\">",
+      {"if": "ddash", "yes": ["-"]}, "-rna</span> and <span class=\"popt\">",
+      {"if": "ddash", "yes": ["-"]}, "-protein</span>. "
   ],
   "all-alph-def": [
     {"if": "meme",
@@ -231,7 +256,7 @@ var shared_doc = {
       ]
     },
     {"if": "ambig-unknown",
-      "yes" : "All ambiguous letters are treated as 'unknown'.",
+      "yes" : "All ambiguous letters are treated as \"unknown\".",
       "no": "",
     }
   ],
@@ -253,7 +278,8 @@ var shared_doc = {
       "yes": "<span class='experimental'>Not recommended</span> ",
       "no": ""
     },
-    "Same as specifying the <span class='popt'>-alph</span> option with the standard <a href='alphabet-format.html#standard_",
+    "Same as specifying the <span class='popt'>",
+    {"if": "ddash", "yes": ["-"]}, "-alph</span> option with the standard <a href='alphabet-format.html#standard_",
     {"if": "dna",
       "yes" : "DNA'>DNA alphabet definition file</a>. ",
       "no" : [
@@ -264,30 +290,36 @@ var shared_doc = {
       ]
     },
     {"if": "ambig-unknown",
-      "yes": "All ambiguous characters are treated as 'unknown'. ",
+      "yes": "All ambiguous characters are treated as \"unknown\". ",
       "no": ""
     }, 
     "Incompatible with options ",
     {"if": "dna", 
-      "yes" : "<span class='popt'>-alph</span>, <span class='popt'>-rna</span>, and <span class='popt'>-protein</span>. ",
+      "yes" : ["<span class='popt'>", {"if": "ddash", "yes": ["-"]}, "-alph</span>, ",
+               "<span class='popt'>", {"if": "ddash", "yes": ["-"]}, "-rna</span>, and ",
+               "<span class='popt'>", {"if": "ddash", "yes": ["-"]}, "-protein</span>. "],
       "no" : [
         {"if": "rna",
-	  "yes" : "<span class='popt'>-alph</span>, <span class='popt'>-dna</span>, and <span class='popt'>-protein</span>. ",
-	  "no" : "<span class='popt'>-alph</span>, <span class='popt'>-dna</span>, and <span class='popt'>-rna</span>. "
+          "yes" : ["<span class='popt'>", {"if": "ddash", "yes": ["-"]}, "-alph</span>, ",
+               "<span class='popt'>", {"if": "ddash", "yes": ["-"]}, "-dna</span>, and ",
+               "<span class='popt'>", {"if": "ddash", "yes": ["-"]}, "-protein</span>. "],
+          "no" : ["<span class='popt'>", {"if": "ddash", "yes": ["-"]}, "-alph</span>, ",
+               "<span class='popt'>", {"if": "ddash", "yes": ["-"]}, "-dna</span>, and ",
+               "<span class='popt'>", {"if": "ddash", "yes": ["-"]}, "-rna</span>."]
         }
       ]
     },
     {"if": "experimental",
-      "yes": "This option is <b>not</b> recommended for use with this program. ",
+      "yes": " This option is <b>not</b> recommended for use with this program. ",
       "no": ""
     },
   ],
   "all-stdalph-def": [
     {"if": "streme",
-      "yes": "The sequences are assumed to be in the DNA alphabet and all ambiguous letters are treated as 'unknown'. ",
+      "yes": "The sequences are assumed to be in the DNA alphabet and all ambiguous letters are treated as \"unknown\". ",
       "no": [
 	{"if": "meme",
-	  "yes": "The sequences are assumed to be in the protein alphabet and all ambiguous letters are treated as 'unknown'. ",
+	  "yes": "The sequences are assumed to be in the protein alphabet and all ambiguous letters are treated as \"unknown\". ",
 	  "no": [
 	    {"if": "ignore",
 	      "yes": "When no alphabet is specified, the sequence content is ignored. ",
@@ -314,6 +346,51 @@ var shared_doc = {
     }
   ],
 
+  /* [x]alph */
+  "all-x_alph-opt": ["[x]alph"],
+  "all-x_alph-val": ["alphabet file"],
+  "all-x_alph-desc": ["The <span class=\"pdat\">alphabet file</span> contains the ",
+    "<a href=\"alphabet-format.html\">alphabet definition</a> of a (possibly non-standard) alphabet. ",
+    "The input <b>sequences</b> must be in this alphabet. ",
+    "If the input <b>motifs</b> are in a different alphabet than the input sequences, ",
+    "specifying <span class='popt'>",
+    {"if": "ddash", "yes": ["-"]},
+    "-xalph</span> will cause the input motifs to be converted to this new alphabet, ",
+    "with the probabilities for the new symbols set to zero prior to applying pseudocounts."
+  ],
+  "all-x_alph-def": [
+     "The sequences are assumed to be in the DNA alphabet and motifs retain the alphabet defined in the motif file.",
+  ],
+
+  /* dna2rna */
+  "all-dna2rna-opt": ["dna2rna"],
+  "all-dna2rna-desc": [
+    "The input DNA sequences will be treated as RNA (treating <code>T</code> as representing <code>U</code>), ",
+    "and the output motifs will use the RNA alphabet. ",
+    "If you provide any known motifs using the ",
+    {"if": "memechip",
+       "yes": [
+         "<span class='popt'>-db</span> "
+       ],
+       "no": [
+         "<span class='popt'>--m</span> "
+       ],
+    },
+    "option, they must use the RNA alphabet or you must also specify ",
+    {"if": "memechip",
+       "yes": [
+         "<span class='popt'>-xalph</span> "
+       ],
+       "no": [
+         "<span class='popt'>--xalph</span> "
+       ],
+    },
+    "with an RNA alphabet file."
+  ],
+  "all-dna2rna-def": [
+    "Treat input DNA sequences as DNA."
+  ],
+
   /* xalph */
   "all-xalph-opt": ["xalph"],
   "all-xalph-val": [
@@ -416,7 +493,7 @@ var shared_doc = {
 
   /* For format2meme conversion scripts, hence the 2meme prefix */
   "2meme-truncate-opt": ["truncate_names"],
-  "2meme-truncate-desc": ["Truncate the motif names (IDs) at the first underscore. (E.g., if the ID is 'Arid3a_primary', then the motif name will be 'Arid3a'. "],
+  "2meme-truncate-desc": ["Truncate the motif names (IDs) at the first underscore. (E.g., if the ID is \"Arid3a_primary\", then the motif name will be \"Arid3a\". "],
   "2meme-truncate-def": ["Use the full motif names. "],
 
   "2meme-numseqs-opt": ["numseqs"],
