@@ -6,14 +6,14 @@
  * VERSION: $Revision: 1.1.1.1 $
  * DESCRIPTION: Some simple array-handling routines.
  ********************************************************************/
+#include <assert.h>
+#include <errno.h>
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <math.h>
 #include <string.h>
-#include <assert.h>
 #include "array.h"
 
-
 /***********************************************************************
  * Allocate one array.
  ***********************************************************************/
@@ -45,6 +45,69 @@ ARRAY_T* resize_array
   return(new_array);
 }
 
+/***********************************************************************
+ * Resize an array.  Allocates the array if it is null. Initilizes values.
+ ***********************************************************************/
+ARRAY_T* resize_init_array
+  (ARRAY_T* array,
+   const int num_items,
+   const ATYPE default_value)
+{
+  ARRAY_T* new_array;
+  int i;
+
+  if (array == NULL) {
+    new_array = allocate_array(num_items);
+    init_array(default_value, array);
+  } else {
+    mm_resize(array->items, num_items, ATYPE);
+    for (i = array->num_items; i < num_items; i++) array->items[i] = default_value;
+    array->num_items = num_items;
+    new_array = array;
+  }
+  return(new_array);
+}
+
+/***********************************************************************
+ * extract_subarray
+ * Extract a sub-array.
+ * Uses half-open, 0 based coordinates [0,num_items)
+ ***********************************************************************/
+ARRAY_T* extract_subarray(ARRAY_T* array, const int start, const int end) {
+  int i, j;
+  if (start >= array->num_items) {
+    die(
+      "Error in extract_array. The staring position (%d)\n"
+      "must be less than the current length of the array (%d).\n",
+      start,
+      array->num_items
+    );
+  }
+  if (end > array->num_items) {
+    die(
+      "Error in extract_array. The end position (%d)\n"
+      "must be less than or equal to the current length of the array (%d).\n",
+      end,
+      array->num_items
+    );
+  }
+  if (end <= start) {
+    die(
+      "Error in extract_array. The end position (%d)\n"
+      "must be greater than the start position (%d).\n",
+      end,
+      array->num_items
+    );
+  }
+
+  ARRAY_T* new_array = allocate_array(end - start);
+  for (i = start, j = 0; i < end; ++i) {
+    set_array_item(j, get_array_item(i, array), new_array);
+    ++j;
+  }
+
+  return(new_array);
+}
 
 
 /**************************************************************************
@@ -52,7 +115,7 @@ ARRAY_T* resize_array
  * dump when attempting to access a null pointer.
  **************************************************************************/
 static void check_null_array
-  (ARRAY_T* array)
+  (const ARRAY_T* array)
 {
 #ifdef BOUNDS_CHECK
   if (array == NULL) {
@@ -89,10 +152,10 @@ static void array_bounds_check
 /***********************************************************************
  * Check to be sure that two arrays have the same length.
  ***********************************************************************/
-static BOOLEAN_T check_array_dimensions
-  (BOOLEAN_T die_on_mismatch,
-   ARRAY_T*  array1,
-   ARRAY_T*  array2)
+static bool check_array_dimensions
+  (bool die_on_mismatch,
+   const ARRAY_T*  array1,
+   const ARRAY_T*  array2)
 {
   /* Check to see that the two arrays are of the same length. */
   if (get_array_length(array1) != get_array_length(array2)) {
@@ -100,9 +163,9 @@ static BOOLEAN_T check_array_dimensions
       die("Arrays have differing lengths (%d != %d).\n", 
 	  get_array_length(array1), get_array_length(array2));
     }
-    return(FALSE);
+    return(false);
   }
-  return(TRUE);
+  return(true);
 }
 
 
@@ -110,7 +173,7 @@ static BOOLEAN_T check_array_dimensions
  * Basic access routines.
  ***********************************************************************/
 int get_array_length
-  (ARRAY_T* array)
+  (const ARRAY_T* array)
 {
   check_null_array(array);
   return(array->num_items);
@@ -138,7 +201,7 @@ void set_array_item_defcheck
   array->items[index] = value;
 }
 
-void incr_array_item
+void incr_array_item_defcheck
   (int      index,
    ATYPE    value,
    ARRAY_T* array)
@@ -265,14 +328,14 @@ void fill_array
  * Copy an array into another array of the same length.
  ***********************************************************************/
 void copy_array
-  (ARRAY_T* source,
+  (const ARRAY_T* source,
    ARRAY_T* dest)
 {
   int num_items;
 
   check_null_array(source);
   check_null_array(dest);
-  check_array_dimensions(TRUE, source, dest);
+  check_array_dimensions(true, source, dest);
 
   num_items = get_array_length(source);
   if (num_items != 0) {
@@ -283,7 +346,7 @@ void copy_array
 /***********************************************************************
  * Determine whether two arrays are equal, within a given bound.
  ***********************************************************************/
-BOOLEAN_T equal_arrays
+bool equal_arrays
   (ATYPE    close_enough,
    ARRAY_T* array1,
    ARRAY_T* array2)
@@ -295,8 +358,8 @@ BOOLEAN_T equal_arrays
   check_null_array(array2);
 
   /* Verify that the arrays are of the same length. */
-  if (!check_array_dimensions(FALSE, array1, array2)) {
-    return(FALSE);
+  if (!check_array_dimensions(false, array1, array2)) {
+    return(false);
   }
 
   /* Check to be sure that each value is the same. */
@@ -304,11 +367,11 @@ BOOLEAN_T equal_arrays
   for (i_item = 0; i_item < num_items; i_item++) {
     if (!almost_equal(get_array_item(i_item, array1)
 		     - get_array_item(i_item, array2), 0.0, close_enough)) {
-      return(FALSE);
+      return(false);
     }
   }
 
-  return(TRUE);
+  return(true);
 }
 
 /***********************************************************************
@@ -323,7 +386,7 @@ void sum_array
   
   check_null_array(array1);
   check_null_array(array2);
-  check_array_dimensions(TRUE, array1, array2);
+  check_array_dimensions(true, array1, array2);
 
   num_items = get_array_length(array1);
   for (i_item = 0; i_item < num_items; i_item++) {
@@ -343,7 +406,7 @@ void element_product
   
   check_null_array(array1);
   check_null_array(array2);
-  check_array_dimensions(TRUE, array1, array2);
+  check_array_dimensions(true, array1, array2);
 
   num_items = get_array_length(array1);
   for (i_item = 0; i_item < num_items; i_item++) {
@@ -363,7 +426,7 @@ double dot_product
   int num_items;
   double return_value;
   
-  check_array_dimensions(TRUE, array1, array2);
+  check_array_dimensions(true, array1, array2);
 
   num_items = get_array_length(array1);
   return_value = 0.0;
@@ -427,6 +490,47 @@ ATYPE array_total
   return(total);
 }
 
+/***********************************************************************
+ * Compute the sum of the elements in a subarray
+ ***********************************************************************/
+ATYPE total_subarray(int start_index, int length, ARRAY_T* array) {
+  ATYPE total;
+  int i;
+
+  /* Compute the total. */
+  total = 0.0;
+  for (i = start_index; i < start_index + length; i++) {
+    total += get_array_item(i, array);
+  }
+  return(total);
+}
+
+/*************************************************************************
+ * Test whether a given array is already sorted.
+ *************************************************************************/
+bool is_sorted
+  (bool good_score_is_low,
+   ARRAY_T*  my_array)
+{
+  ATYPE prev_value = get_array_item(0, my_array);
+  int index = 0;
+  int length = get_array_length(my_array);
+  for (index = 1; index < length; index++) {
+    ATYPE this_value = get_array_item(index, my_array);
+    if (good_score_is_low) {
+      if (this_value < prev_value) {
+	return(false);
+      }
+    } else {
+      if (this_value > prev_value) {
+	return(false);
+      }
+    }      
+    prev_value = this_value;
+  }
+  return(true);
+}
+
 /***********************************************************************
  * Sort the elements in an array. 
  ***********************************************************************/
@@ -476,7 +580,7 @@ static int reverse_sort_compare
 }
 
 void sort_array
-  (BOOLEAN_T reverse_sort,
+  (bool reverse_sort,
    ARRAY_T* array)
 {
   if (reverse_sort) {
@@ -538,7 +642,7 @@ ATYPE sum_of_square_diffs
   
   check_null_array(array1);
   check_null_array(array2);
-  check_array_dimensions(TRUE, array1, array2);
+  check_array_dimensions(true, array1, array2);
 
   num_items = get_array_length(array1);
   for (i_item = 0; i_item < num_items; i_item++) {
@@ -570,7 +674,7 @@ ATYPE compute_median
   ATYPE return_value;
 
   /* Sort the array. */
-  sort_array(FALSE, array);
+  sort_array(false, array);
 
   /* If there are an odd number of elements, return the middle one. */
   num_items = get_array_length(array);
@@ -610,6 +714,46 @@ void read_array
   }
 }
 
+/***********************************************************************
+ * Read an array of unknown length from a file.
+ * Caller is resposbile for freeing array.
+ ***********************************************************************/
+ARRAY_T *read_array_from_file(const char* filename) {
+
+  int array_size = 100;
+  int i_item = 0;
+  int num_read = 0;
+  ATYPE value;
+  
+  FILE *array_file = fopen(filename, "r");
+  if (array_file == NULL) {
+    die(
+      "Unable to open file: %s.\nError message: %s.\n", 
+      filename, 
+      strerror(errno)
+    );
+  }
+
+  ARRAY_T *array = allocate_array(array_size);
+  while ((num_read = fscanf(array_file, ASCAN, &value)) == 1) {
+    set_array_item(i_item, value, array);
+    ++i_item;
+    if (i_item >= array_size) {
+      resize_array(array, 2 * array_size);
+      array_size = 2 *array_size;
+    }
+  }
+
+  if (num_read == 0) {
+    die("Error reading array at position %d.\n", i_item);
+  }
+  fclose(array_file);
+
+  resize_array(array, i_item);
+
+  return array;
+}
+
 /***********************************************************************
  * Write an array to a file.
  ***********************************************************************/
@@ -617,7 +761,7 @@ void print_array
   (ARRAY_T*  array,         /* The array to be printed. */
    int       width,         /* Width of each cell. */
    int       precision,     /* Precision of each cell. */
-   BOOLEAN_T eol,           /* Include an EOL char at the end? */
+   bool eol,           /* Include an EOL char at the end? */
    FILE*     outfile)       /* File to which to write. */
 {
   int   i_item;
@@ -630,6 +774,7 @@ void print_array
   for (i_item = 0; i_item < num_items; i_item++) {
     item = get_array_item(i_item, array);
     fprintf(outfile, APRINT, width, precision, item);
+    fputc('\n', outfile);
   }
   if (eol) {
     fprintf(outfile, "\n");
@@ -645,7 +790,7 @@ void print_sub_array
    ARRAY_T*  array,         /* The array to be printed. */
    int       width,         /* Width of each cell. */
    int       precision,     /* Precision of each cell. */
-   BOOLEAN_T eol,           /* Include an EOL char at the end? */
+   bool eol,           /* Include an EOL char at the end? */
    FILE*     outfile)       /* File to which to write. */
 {
   int   i_item;
@@ -700,7 +845,7 @@ ARRAY_T* bootstrap_array
   int i_item;
   int num_inputs = get_array_length(source_array);
   for (i_item = 0; i_item < num_items; i_item++) {
-    int random_index = (int)(my_drand() * (double)(num_inputs));
+    int random_index = (int)(drand_mt() * (double)(num_inputs));
     ATYPE random_item = get_array_item(random_index, source_array);
     set_array_item(i_item, random_item, return_array);
   }
@@ -722,7 +867,7 @@ void dot_divide
   
   check_null_array(array1);
   check_null_array(array2);
-  check_array_dimensions(TRUE, array1, array2);
+  check_array_dimensions(true, array1, array2);
 
   num_items = get_array_length(array1);
   for (i_item = 0; i_item < num_items; i_item++) {
@@ -980,7 +1125,7 @@ void log_normalize
  * Convert a given array to or from logs.
  **************************************************************************/
 void convert_to_from_log_array
-  (BOOLEAN_T to_log,
+  (bool to_log,
    ARRAY_T*  source_array,
    ARRAY_T*  target_array)
 {
@@ -1019,7 +1164,7 @@ void mix_log_arrays
   check_null_array(array2);
 
   /* Verify that the arrays are of the same length. */
-  check_array_dimensions(TRUE, array1, array2);
+  check_array_dimensions(true, array1, array2);
 
   /* Verify that we've got a reasonable mixing parameter. */
   if ((mixing > 1.0) || (mixing < 0.0)) {
@@ -1051,7 +1196,7 @@ void randomize_array
 
   num_items = get_array_length(array);
   for (i_item = 0; i_item < num_items; i_item++) {
-    set_array_item(i_item, my_drand() * magnitude, array);
+    set_array_item(i_item, drand_mt() * magnitude, array);
   }
 }
 
@@ -1070,7 +1215,7 @@ void add_noise
 
   num_items = get_array_length(array);
   for (i_item = 0; i_item < num_items; i_item++) {
-    noise = magnitude * (2 * my_drand() - 1);
+    noise = magnitude * (2 * drand_mt() - 1);
     incr_array_item(i_item, noise, array);
   }
 }
