@@ -28,7 +28,6 @@
 #include "log-hmm.h"        // Convert an HMM to or from log form.
 #include "match.h"          // Find sequence to model match from traceback
 #include "metameme.h"       // Global Meta-MEME functions.
-#include "meme-io.h"        // Read motifs from MEME output.
 #include "mhmm.h"           // Scan a sequence db with HMM.
 #include "mhmm-state.h"     // Data structure for HMMs.
 #include "mhmms.h"          // Scan a sequence db with HMM.
@@ -41,6 +40,7 @@
 #include "string-list.h"    // List of strings (for motif IDs).
 #include "utils.h"          // Generic utilities.
 #include "write-mhmm.h"     // Functions for producing MHMM output.
+#include "motif-in.h"
 
 #define SLOP 1E-5
 
@@ -70,7 +70,7 @@ void fix_trans_in(MHMM_T* the_hmm) {
     for (i_row = 0; i_row < n; i_row++) {
       double p = get_matrix_cell(i_row, i_col, trans);	// The transition probability.
       int m = 0;
-      if (!is_zero(p, FALSE)) {
+      if (!is_zero(p, false)) {
         MHMM_STATE_T * in_state = &(the_hmm->states[i_col]);
         assert(m < in_state->ntrans_in);
         set_array_item(m, p, in_state->trans_in);
@@ -84,81 +84,52 @@ void fix_trans_in(MHMM_T* the_hmm) {
 /*************************************************************************
  *  Build a linear HMM model for the motifs
  *************************************************************************/
-MHMM_T* build_linear_model(struct options* options) {
+MHMM_T* build_linear_model(OPTIONS_T *options) {
 
   int num_motifs = 0;              // The number of motifs in the model.
-  MOTIF_T  motifs[2 * MAX_MOTIFS]; // The motifs.
-  BOOLEAN_T has_reverse_strand = FALSE; // MEME file contained both strands
+  RBTREE_T  *motifs; // The motifs.
+  bool has_reverse_strand = false; // MEME file contained both strands
   ARRAY_T* background = NULL;      // Background probs for alphabet.
   ORDER_T* order_spacing = NULL;   // Linear HMM order and spacing.
   MATRIX_T* transp_freq = NULL;    // Matrix of inter-motif transitions freqs.
   MATRIX_T* spacer_ave = NULL;     // Matrix of average spacer lengths.
   MHMM_T* the_hmm = NULL;          // The HMM being constructed.
-  STRING_LIST_T* motif_occurrences = NULL; /* Strings describing occurrences
-                                             of motifs */
 
   // Gotta have positive number of spacer states.
   if (options->spacer_states <= 0) {
     die("Negative spacer length (%d).\n", options->spacer_states);
   }
 
-  // Make sure motifs weren't selected redundantly.
-  if ((get_num_strings(options->requested_motifs) != 0)
-      && (options->order_string != NULL)) {
-    die("Can't combine --motif and --order options.");
-  }
-
-  // Parse the order string.
-  order_spacing = create_order(options->order_string);
-
   /**********************************************
    * READING THE MOTIFS
    **********************************************/
-  read_meme_file(
-		 options->meme_filename,
-		 options->bg_filename,
-		 options->motif_pseudo,
-		 &num_motifs,
-		 motifs,
-		 &motif_occurrences,
-		 &has_reverse_strand,
-		 &background
-		 );
-
-  // If motifs use protein alphabet we will not scan both strands
-  ALPH_T alphabet_type = which_alphabet();
-  if (alphabet_type == PROTEIN_ALPH) {
-    options->alphabet = PROTEIN_ALPH;
-  }
-  else {
-    options->alphabet = DNA_ALPH;
-  }
-
-  process_raw_motifs_for_model(
-       &num_motifs,
-       motifs,
-       motif_occurrences,
-       options->requested_motifs,
-       has_reverse_strand,
-       FALSE, // keep unused motifs
-       options->motif_p_threshold,
-       options->motif_e_threshold,
-       0.0, // complexity threshold
-       &order_spacing,
-       &transp_freq,
-       &spacer_ave,
-       options->trans_pseudo,
-       options->spacer_pseudo
-  );
+  load_filter_process_motifs_for_hmm(
+      0, // no top n motifs
+      options->request_nums,
+      options->request_ids,
+      options->motif_e_threshold,
+      0.0, // no complexity threshold
+      options->order_string,
+      options->motif_p_threshold, // for filtering occurrence list
+      options->meme_filename,
+      options->bg_filename,
+      options->motif_pseudo,
+      options->trans_pseudo,
+      options->spacer_pseudo,
+      false, // keep unused motifs
+      &(options->alphabet),
+      &motifs,
+      &background,
+      &order_spacing,
+      &transp_freq,
+      &spacer_ave
+      );
 
   /**********************************************
    * BUILDING THE HMM
    **********************************************/
 
-  if (order_spacing != NULL) {
-    reorder_motifs(order_spacing, &num_motifs, motifs);
-  }
-  else {
+  if (order_spacing == NULL) {
     die("No order specified for the motifs.\n"
         "For the linear model the motif file must contain motif occurence\n"
         "data or the motif order must be specified using the --order option.");
@@ -169,21 +140,16 @@ MHMM_T* build_linear_model(struct options* options) {
     order_spacing,
     options->spacer_states,
     motifs,
-    num_motifs,
     options->fim,
     &the_hmm
   );
   fix_trans_in(the_hmm);
 
-  free_string_list(motif_occurrences);
   free_array(background);
   free_order(order_spacing);
   free_matrix(transp_freq);
   free_matrix(spacer_ave);
-  int i_motif = 0;
-  for (i_motif = 0; i_motif < num_motifs; i_motif++) {
-    free_motif(&(motifs[i_motif]));
-  }
+  rbtree_destroy(motifs);
 
   // Add some global information.
   copy_string(&(the_hmm->motif_file), options->meme_filename);
@@ -195,14 +161,14 @@ MHMM_T* build_linear_model(struct options* options) {
 
 void search_with_model(
  MHMM_T* the_mhmm,
- BOOLEAN_T* got_evd,
+ bool* got_evd,
  OPTIONS_T *options,
  PATTERN_T *pattern
 ) {
 
-  BOOLEAN_T viterbi = FALSE;
-  BOOLEAN_T use_pvalues = FALSE;
-  BOOLEAN_T both_strands = FALSE;
+  bool viterbi = false;
+  bool use_pvalues = false;
+  bool both_strands = false;
   EVD_SET evd_set;      // EVD data structure
   evd_set.evds = NULL;  // Initialize pointer to evds.
 
@@ -212,14 +178,14 @@ void search_with_model(
 
   // Figure out what kind of scoring to do.
   if (options->paths == SINGLE) {
-    viterbi = TRUE;
+    viterbi = true;
   } else {
-    viterbi = FALSE;
+    viterbi = false;
   }
 
   // Force p-value scoring if p-value threshold given.
   if (options->p_threshold != 0) {
-    use_pvalues = TRUE;
+    use_pvalues = true;
   }
 
   // Check p-threshold is in range [0<p<=1].
@@ -229,15 +195,16 @@ void search_with_model(
 
   // Set the PAM distance.
   if (options->pam_distance == -1) {
-    options->pam_distance = (which_alphabet() == PROTEIN_ALPH)
-      ? DEFAULT_PROTEIN_PAM : DEFAULT_DNA_PAM;
+    options->pam_distance = 
+       (alph_is_builtin_protein(the_mhmm->alph) ? DEFAULT_PROTEIN_PAM : DEFAULT_DNA_PAM);
   }
-  int beta = (which_alphabet() == PROTEIN_ALPH) ? 10 : 1;
+  //int beta = (options->alphabet == PROTEIN_ALPH) ? 10 : 1;
+  int beta = alph_is_builtin_protein(the_mhmm->alph) ? 10 : 1;
 
   // Convert the model to log space.
-  MHMM_T*   the_log_hmm;      // The HMM, with probs converted to logs.
+  MHMM_T* the_log_hmm = allocate_mhmm(the_mhmm->alph, the_mhmm->num_states);
   convert_to_from_log_hmm(
-    TRUE, // Convert to logs.
+    true, // Convert to logs.
     options->zselo,
     options->gap_open,
     options->gap_extend,
@@ -246,7 +213,7 @@ void search_with_model(
     options->pam_distance,
     beta,
     the_mhmm,
-    &the_log_hmm
+    the_log_hmm
   );
 
   // Set up PSSM matrices if doing motif_scoring
@@ -257,7 +224,9 @@ void search_with_model(
     options->p_threshold,
     both_strands,
     options->allow_weak_motifs,
-    the_log_hmm
+    the_log_hmm,
+    NULL,	// PSP
+    1		// PSP alpha
   );
 
   //
@@ -267,7 +236,7 @@ void search_with_model(
     options->p_threshold,
     -1,    // dp_threshold,
     -1,    // max_gap
-    FALSE, // negatives_only
+    false, // negatives_only
     the_log_hmm
   );
 
@@ -280,7 +249,7 @@ void search_with_model(
   if (open_file(
         options->seq_filename,
         "r",
-        TRUE,
+        true,
         "sequence",
         "sequences",
         &seq_file
@@ -288,7 +257,7 @@ void search_with_model(
     exit(1);
   }
 
-  double start_time = myclock();
+  double start_time = mytime(0);
   double old_loop_time = start_time;
   double seconds_since_last_report = 0.0;
 
@@ -302,7 +271,7 @@ void search_with_model(
   int num_seqs = 0;
 
   this_match = allocate_match(); // This sequence-to-model match.
-  while (read_one_fasta(seq_file, MAX_SEQ, &sequence)) {
+  while (read_one_fasta(options->alphabet, seq_file, MAX_SEQ, &sequence)) {
 
     num_seqs++;
 
@@ -326,7 +295,7 @@ void search_with_model(
     }
 
     // Convert the sequence to alphabet-specific indices.
-    prepare_sequence(sequence);
+    prepare_sequence(sequence, options->alphabet);
     // Sequeence length may have changed
     seq_length = get_seq_length(sequence);
     assert(get_seq_char(seq_length, sequence) == '\0');
@@ -359,6 +328,9 @@ void search_with_model(
         options->p_threshold,
         get_int_sequence(sequence),
         seq_length,
+        NULL,		// PSP
+        0,		// num_priors
+	0,		// PSP ALPHA
         the_log_hmm,
         &motif_score_matrix
       );
@@ -371,7 +343,7 @@ void search_with_model(
         get_int_sequence(sequence),
         seq_length,
         the_log_hmm,
-        TRUE,  // Save Viterbi path?
+        true,  // Save Viterbi path?
         motif_score_matrix,
         dp_matrix,
         trace_matrix,
@@ -383,7 +355,7 @@ void search_with_model(
         get_int_sequence(sequence),
         seq_length,
         the_log_hmm,
-        //motif_score_matrix, // FIXME
+        //motif_score_matrix, // TODO
         dp_matrix,
         this_match
       );
@@ -391,10 +363,10 @@ void search_with_model(
 
     // Store the score, ID, length and comment for later printing.
     //assert(get_match_seq_length(this_match) == get_seq_length(sequence));
-    //assert(get_match_seq_length(this_match) == get_seq_length(sequence));
     store_sequence(
+      options->alphabet,
       options->motif_scoring,
-      FALSE, // Don't print in mhmmscan format.
+      false, // Don't print in mhmmscan format.
       DEFAULT_OUTPUT_WIDTH,
       sequence,
       this_match,
@@ -404,10 +376,11 @@ void search_with_model(
       score_set,
       *got_evd,
       evd_set,
-      FALSE, // print fancy
+      //false, // store the Viterbi trace
+      true, // store the Viterbi trace
       the_log_hmm,
       motif_score_matrix,
-      FALSE, // Store GFF?,
+      false, // Store GFF?,
       scanned_seq
     );
 
@@ -416,13 +389,13 @@ void search_with_model(
      * descriptions of low-scoring sequences to not be stored.  The
      * distribution will be recomputed using all saved scores when all
      * sequences have been read.  */
-    if (score_set->n == EVD_NUM_SEQS && *got_evd == FALSE) {
+    if (score_set->n == EVD_NUM_SEQS && *got_evd == false) {
       evd_set = calc_distr(
-        *score_set,   // Set of scores.
-        use_pvalues, // Use exponential distribution?
-        FALSE         // Use sequence E-values.
+        *score_set,			// Set of scores.
+        use_pvalues ? D_EXP : D_EVD,	// Use exponential distribution?
+        false				// Use sequence E-values.
       );
-      if (evd_set.n > 0) *got_evd = TRUE;
+      if (evd_set.n > 0) *got_evd = true;
     }
 
     // Free the memory used by this sequence.
@@ -431,7 +404,7 @@ void search_with_model(
 
     // If requested, periodically report progress
     if (options->progress > 0.0) {
-      double new_loop_time = myclock();
+      double new_loop_time = mytime(0);
       seconds_since_last_report = (new_loop_time - old_loop_time) / 1e6;
       if (seconds_since_last_report > options->progress) {
         double elapsed_seconds = (new_loop_time - start_time) / 1e6;
@@ -446,8 +419,6 @@ void search_with_model(
     }
   }
 
-  //double end_time = myclock();
-
   /***********************************************
    * Calculate the E-values and store them as the keys.
    ***********************************************/
@@ -455,9 +426,9 @@ void search_with_model(
   // If successful, calculate the E-values and
   // store them as keys.
   evd_set = calc_distr(
-    *score_set,   // Set of scores.
-    use_pvalues, // Use exponential distribution?
-    FALSE         // Use sequence E-values.
+    *score_set,				// Set of scores.
+    use_pvalues ? D_EXP : D_EVD,	// Use exponential distribution?
+    false         			// Use sequence E-values.
   );
   if (evd_set.n > 0) {
     int q, t, N;
@@ -466,9 +437,9 @@ void search_with_model(
     N = evd_set.non_outliers;    // Use number of non-outliers.
     evd_set.N = N;
     calc_evalues(&evd_set, N, q, t);
-    *got_evd = TRUE;
+    *got_evd = true;
   }
-  evd_set.negatives_only = FALSE;  // Used real sequences.
+  evd_set.negatives_only = false;  // Used real sequences.
 
   /* Tie up loose ends. */
   myfree(evd_set.evds);
@@ -559,14 +530,18 @@ static  void print_settings_xml(
   fprintf(out, "<setting name=\"%s\">%3.2g</setting>\n", "spacer pseudocount", options->spacer_pseudo);
   fprintf(out, "<setting name=\"%s\">%3.2g</setting>\n", "transition pseudocount", options->trans_pseudo);
   fprintf(out, "<setting name=\"%s\">%d</setting>\n", "verbosity", verbosity);
-  int i = 0;
-  int num_strings = get_num_strings(options->requested_motifs);
-  for(i = 0; i < num_strings; i++) {
-    fprintf(
-      out,
-      "<setting name=\"%s\">%s</setting>\n", "requested motif",
-      get_nth_string(i, options->requested_motifs)
-    );
+  RBNODE_T *node;
+  for(node = rbtree_first(options->request_nums); node != NULL; node = rbtree_next(node)) {
+    int motif_num;
+    int strands;
+    motif_num = *((int*)rbtree_key(node));
+    strands = *((int*)rbtree_value(node));
+    if (strands & MOTIF_PLUS) {
+      fprintf(out, "<setting name=\"%s\">+%d</setting>\n", "requested motif", motif_num);
+    }
+    if (strands & MOTIF_MINUS) {
+      fprintf(out, "<setting name=\"%s\">-%d</setting>\n", "requested motif", motif_num);
+    }
   }
 
   fputs("</settings>\n",  out);
@@ -582,6 +557,7 @@ static void print_beadstring_xml_file(
   ARRAY_T *background,
   char  *stylesheet
 ) {
+  int i;
 
   fputs("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n", out);
   if (stylesheet != NULL) {
@@ -589,14 +565,7 @@ static void print_beadstring_xml_file(
   }
   fputs("<!-- Begin document body -->\n", out);
 
-  const char *archive_date = ARCHIVE_DATE;
-  int i = strlen(archive_date);
-  fprintf(out,
-    "<beadstring version=\"%s\" release=\"%.*s\">\n",
-    VERSION,
-    i,
-    archive_date
-  );
+  fputs("<beadstring version=\"" VERSION "\" release=\"" ARCHIVE_DATE "\">\n", out);
   fputs("  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"", out);
   fputs("\n", out);
   fputs("  xsi:schemaLocation=", out);
@@ -605,22 +574,24 @@ static void print_beadstring_xml_file(
   fputs("  xmlns:bstr=\"http://noble.gs.washington.edu/schema/beadstring\"\n>\n", out);
   fprintf(out, "<command-line>%s</command-line>\n", options->command_line);
   print_settings_xml(out, options);
+  //TODO make work with other alphabets
   fprintf(
     out,
     "<alphabet>%s</alphabet>\n",
-    options->alphabet == DNA_ALPH ? "nucleotide" : "protein"
+    alph_is_builtin_dna(options->alphabet) ? "nucleotide" : "protein"
   );
   fprintf(
     out,
     "<background source=\"%s\">\n",
     options->bg_filename ? options->bg_filename : "non-redundant database"
   );
-  int alph_size = get_alph_size(ALPH_SIZE);
-  for (i = 0; i < alph_size; i++) {
+  //int asize = alph_size(options->alphabet, ALPH_SIZE);
+  int asize = alph_size_core(options->alphabet);
+  for (i = 0; i < asize; i++) {
     fprintf(
       out,
       "<value letter=\"%c\">%1.3f</value>\n",
-      get_alph_char(i),
+      alph_char(options->alphabet, i),
       get_array_item(i, background)
     );
   }
@@ -634,7 +605,7 @@ static void print_beadstring_xml_file(
 
 /**********************************************************************
  * This function saves the sequences containing a hit to a motif
- * which is greater then the output p/q-value threshold. If the sequence
+ * which is greater than the output p/q-value threshold. If the sequence
  * is less then 10k in length, a copy of the complete sequence will be
  * saved.
  *********************************************************************/
@@ -648,15 +619,15 @@ void print_sequences(FILE *out, OPTIONS_T *options, CISML_T *cisml) {
 
   // Open the FASTA file for reading.
   FILE* fasta_file = NULL;
-  if (open_file(options->seq_filename, "r", TRUE, "FASTA", "sequences", &fasta_file) == 0) {
+  if (open_file(options->seq_filename, "r", true, "FASTA", "sequences", &fasta_file) == 0) {
     die("Couldn't open the file %s.\n", options->seq_filename);
   }
 
   // Read the FASTA file one sequence at a time.
-  while (read_one_fasta(fasta_file, MAX_SEQ, &sequence)) {
+  while (read_one_fasta(options->alphabet, fasta_file, MAX_SEQ, &sequence)) {
     char *seq_name = get_seq_name(sequence);
     int seq_length = get_seq_length(sequence);
-    // FIXME Only print those sequences that contain a matched element
+    // TODO Only print those sequences that contain a matched element
     if (seq_length <= max_size_recorded) {
       // If sequence is less then limit record raw sequence
       char *raw_sequence = get_raw_sequence(sequence);
@@ -728,7 +699,7 @@ static void create_hmm_diagram_files(char *mhmm_xml_path, char *mhmm_ps_path,  c
  ***********************************************************************/
 void print_beadstring_results(
   MHMM_T* the_hmm,
-  BOOLEAN_T got_evd,
+  bool got_evd,
   OPTIONS_T* options,
   CISML_T *cisml
 ) {
@@ -737,7 +708,7 @@ void print_beadstring_results(
   if (create_output_directory(
        options->output_dirname,
        options->allow_clobber,
-       FALSE /* Don't print warning messages */
+       false /* Don't print warning messages */
       )
     ) {
     // Failed to create output directory.
@@ -763,7 +734,8 @@ void print_beadstring_results(
     // Create the paths to the output files
     const char *HTML_STYLESHEET = "beadstring.xsl";
     const char *CSS_STYLESHEET = "cisml.css";
-    const char *GFF_STYLESHEET = "cisml-to-gff.xsl";
+    //const char *GFF_STYLESHEET = "cisml-to-gff.xsl";
+    const char *GFF_STYLESHEET = "cisml-to-gff3.xsl";
     const char *TEXT_STYLESHEET = "cisml-to-text.xsl";
     const char *SEQUENCE_FILENAME = "matched_sequences.xml";
     const char *CISML_FILENAME = "cisml.xml";
@@ -772,10 +744,10 @@ void print_beadstring_results(
     const char *GFF_FILENAME = "beadstring.gff";
     char *sequence_path = make_path_to_file(options->output_dirname, SEQUENCE_FILENAME);
     char *cisml_path = make_path_to_file(options->output_dirname, CISML_FILENAME);
-    char *html_stylesheet_path = make_path_to_file(ETC_DIR, HTML_STYLESHEET);
-    char *css_stylesheet_path = make_path_to_file(ETC_DIR, CSS_STYLESHEET);
-    char *text_stylesheet_path = make_path_to_file(ETC_DIR, TEXT_STYLESHEET);
-    char *gff_stylesheet_path = make_path_to_file(ETC_DIR, GFF_STYLESHEET);
+    char *html_stylesheet_path = make_path_to_file(get_meme_data_dir(), HTML_STYLESHEET);
+    char *css_stylesheet_path = make_path_to_file(get_meme_data_dir(), CSS_STYLESHEET);
+    char *text_stylesheet_path = make_path_to_file(get_meme_data_dir(), TEXT_STYLESHEET);
+    char *gff_stylesheet_path = make_path_to_file(get_meme_data_dir(), GFF_STYLESHEET);
     char *html_path = make_path_to_file(options->output_dirname, HTML_FILENAME);
     char *text_path = make_path_to_file(options->output_dirname, TEXT_FILENAME);
     char *gff_path = make_path_to_file(options->output_dirname, GFF_FILENAME);
@@ -814,7 +786,7 @@ void print_beadstring_results(
     if (!cisml_file) {
       die("Couldn't open file %s for output.\n", cisml_path);
     }
-    print_cisml(cisml_file, cisml, TRUE, NULL, TRUE);
+    print_cisml(cisml_file, cisml, true, NULL, true);
     fclose(cisml_file);
 
     // Output HTML
@@ -854,16 +826,16 @@ void print_beadstring_results(
 
     /*
     sort_and_print_scores(
-      FALSE, // print fancy
-      TRUE,  // print_header,
+      false, // print fancy
+      true,  // print_header,
       got_evd,
       options->motif_scoring,
-      FALSE, // Don't print in mhmmscan format.
+      false, // Don't print in mhmmscan format.
       options->max_seq, // Maximum number of sequences to print
       DEFAULT_OUTPUT_WIDTH,
       options->e_threshold,
-      TRUE, // Sort output
-      NULL, // FIXME: GFF output?
+      true, // Sort output
+      NULL, // TODO: GFF output?
       text_file
     );
     */
@@ -932,7 +904,7 @@ static void process_command_line(
 
   // Define the usage message.
   char usage[] =
-    "USAGE: beadstring [options] <motif file> <sequence file>\n"
+    "Usage: beadstring [options] <motif file> <sequence file>\n"
      "\n"
      "   Options related to input and output:\n"
      "     --bgfile <background file>\n"
@@ -978,13 +950,14 @@ static void process_command_line(
   options->seq_filename = NULL;
   options->model_filename = NULL;
   options->order_string = NULL;
-  options->allow_weak_motifs = FALSE;
-  options->allow_clobber = TRUE;
-  options->fim = FALSE;
-  options->local_scoring = TRUE;
-  options->motif_scoring = TRUE; // Always do this (not really an option).
-  options->synth = FALSE;
-  options->zselo = FALSE;
+  options->allow_weak_motifs = false;
+  options->allow_clobber = true;
+  options->fim = false;
+  options->local_scoring = true;
+  //options->motif_scoring = true; // Always do this (not really an option).
+  options->motif_scoring = false;
+  options->synth = false;
+  options->zselo = false;
   options->max_seq = -1;
   options->pam_distance = -1;
   options->paths = SINGLE;
@@ -993,9 +966,10 @@ static void process_command_line(
   options->gap_open = -1.0;
   options->e_threshold = 0.01;
   options->p_threshold = 0.0;
-  options->perform_search = TRUE;
+  options->perform_search = true;
   options->progress = 0.0;
-  options->requested_motifs = new_string_list();
+  options->request_nums = rbtree_create(rbtree_intcmp, rbtree_intcpy, free, rbtree_intcpy, free);
+  options->request_ids = rbtree_create(rbtree_strcmp, rbtree_strcpy, free, rbtree_intcpy, free);
   options->motif_e_threshold = 0.0;
   options->motif_p_threshold = 0.0;
   options->motif_pseudo = DEFAULT_MOTIF_PSEUDO;
@@ -1031,13 +1005,13 @@ static void process_command_line(
     } else if (strcmp(option_name, "model-file") == 0) {
       options->model_filename = option_value;
     } else if (strcmp(option_name, "no-search") == 0) {
-      options->perform_search = FALSE;
+      options->perform_search = false;
     } else if (strcmp(option_name, "o") == 0) {
       options->output_dirname = option_value;
-      options->allow_clobber = FALSE;
+      options->allow_clobber = false;
     } else if (strcmp(option_name, "oc") == 0) {
       options->output_dirname = option_value;
-      options->allow_clobber = TRUE;
+      options->allow_clobber = true;
     } else if (strcmp(option_name, "progress") == 0) {
       options->progress = atof(option_value);
     } else if (strcmp(option_name, "score-file") == 0) {
@@ -1046,7 +1020,19 @@ static void process_command_line(
       verbosity = (VERBOSE_T)atoi(option_value);
 
     } else if (strcmp(option_name, "motif") == 0) {
-      add_string(option_value, options->requested_motifs);
+      if (!store_requested_num(options->request_nums, option_value)) {
+        die("Error processing \"-motif %s\". "
+            "Option value should be a (non-zero) motif position. "
+            "For example the first motif in the file has the "
+            "position 1 and it's reverse complement has the position -1.\n", 
+            option_value);
+      }
+    } else if (strcmp(option_name, "motif-id") == 0) {
+      if (!store_requested_id(options->request_ids, option_value)) {
+        die("Error processing \"-motif-id %s\"."
+            "Option value should be a motif identifer and hence "
+            "not just be a + or a -.\n", option_value);
+      }
     } else if (strcmp(option_name, "moitf-e-thresh") == 0) {
       options->motif_e_threshold = atof(option_value);
     } else if (strcmp(option_name, "motif-p-thresh") == 0) {
@@ -1055,7 +1041,7 @@ static void process_command_line(
       options->order_string = option_value;
 
     } else if (strcmp(option_name, "fim") == 0) {
-      options->fim = TRUE;
+      options->fim = true;
     } else if (strcmp(option_name, "gap-extend") == 0) {
       options->gap_extend = atof(option_value);
     } else if (strcmp(option_name, "gap-open") == 0) {
@@ -1069,14 +1055,14 @@ static void process_command_line(
     } else if (strcmp(option_name, "trans-pseudo") == 0) {
       options->trans_pseudo = atof(option_value);
     } else if (strcmp(option_name, "zselo") == 0) {
-      options->zselo = TRUE;
+      options->zselo = true;
 
     } else if (strcmp(option_name, "allow-weak-motifs") == 0) {
-      options->allow_weak_motifs =  TRUE;
+      options->allow_weak_motifs =  true;
     } else if (strcmp(option_name, "global") == 0) {
-      options->local_scoring = FALSE;
+      options->local_scoring = false;
     } else if (strcmp(option_name, "motif-scoring") == 0) {
-      options->motif_scoring = TRUE;
+      options->motif_scoring = true;
     } else if (strcmp(option_name, "pam") == 0) {
       options->pam_distance = atoi(option_value);
     } else if (strcmp(option_name, "paths") == 0) {
@@ -1088,7 +1074,7 @@ static void process_command_line(
       }
       else {
         fprintf(stderr, "Invalid path option: %s.\n", option_value);
-        fprintf(stderr, usage);
+        fprintf(stderr, "%s", usage);
         exit(1);
       }
     } else if (strcmp(option_name, "p-score") == 0) {
@@ -1099,7 +1085,7 @@ static void process_command_line(
   // Read the two required arguments.
   if (option_index + 2 > argc) {
     fprintf(stderr, "Missing required arguments.\n");
-    fprintf(stderr, usage);
+    fprintf(stderr, "%s", usage);
     exit(1);
   }
   options->meme_filename = argv[option_index];
@@ -1131,6 +1117,7 @@ static void process_command_line(
 int main(int argc, char *argv[]) {
 
   OPTIONS_T options;
+  RBNODE_T *node;
   MHMM_T *the_hmm = NULL;
 
   process_command_line(argc, argv, &options);
@@ -1170,7 +1157,7 @@ int main(int argc, char *argv[]) {
   add_cisml_pattern(cisml, pattern);
 
   // Perform the actual search.
-  BOOLEAN_T got_evd = FALSE;
+  bool got_evd = false;
   if (options.perform_search) {
     search_with_model(the_hmm, &got_evd, &options, pattern);
   }
@@ -1180,8 +1167,29 @@ int main(int argc, char *argv[]) {
   // Print the program parameters.
   printf("Program parameters for beadstring\n");
   printf("  MEME file: %s\n", options.meme_filename);
-  printf("  Motifs:");
-  write_string_list(" ", options.requested_motifs, stdout);
+  printf("  Motif nums:");
+  for (node = rbtree_first(options.request_nums); node != NULL; node = rbtree_next(node)) {
+    int motif_num = *((int*)rbtree_key(node));
+    int strands = *((int*)rbtree_value(node));
+    if (strands & MOTIF_PLUS) {
+      printf(" +%d", motif_num);
+    }
+    if (strands & MOTIF_MINUS) {
+      printf(" -%d", motif_num);
+    }
+  }
+  printf("\n");
+  printf("  Motif IDs:");
+  for (node = rbtree_first(options.request_ids); node != NULL; node = rbtree_next(node)) {
+    char *motif_id = (char*)rbtree_key(node);
+    int strands = *((int*)rbtree_value(node));
+    if (strands & MOTIF_PLUS) {
+      printf(" +%s", motif_id);
+    }
+    if (strands & MOTIF_MINUS) {
+      printf(" -%s", motif_id);
+    }
+  }
   printf("\n");
   printf("  Model topology: linear\n");
   printf("  States per spacer: %d\n", options.spacer_states);
@@ -1192,8 +1200,9 @@ int main(int argc, char *argv[]) {
   printf("\n");
 
   free_mhmm(the_hmm);
-  free_string_list(options.requested_motifs);
   myfree(options.command_line);
+  rbtree_destroy(options.request_nums);
+  rbtree_destroy(options.request_ids);
 
   return(0);
 }
