@@ -24,7 +24,9 @@
 #include "xml-util.h"
 
 typedef struct cisml CISML_T;
+typedef struct cisml_match_it CISML_MATCH_IT_T;
 typedef struct multi_pattern MULTI_PATTERN_T;
+typedef struct multi_pattern_match MULTI_PATTERN_MATCH_T;
 typedef struct pattern PATTERN_T;
 typedef struct scanned_sequence SCANNED_SEQUENCE_T;
 typedef struct matched_element MATCHED_ELEMENT_T;
@@ -37,9 +39,10 @@ typedef struct matched_element MATCHED_ELEMENT_T;
   Other fields set to 0 or NULL.
 **********************************************************************/
 CISML_T *allocate_cisml(
-  char *program_name,
-  char *pattern_file,
-  char *sequence_file
+  const char *program_name,
+  const char *command_line,
+  const char *pattern_file,
+  const char *sequence_file
 );
 
 /**********************************************************************
@@ -56,6 +59,13 @@ void free_cisml(CISML_T *cisml);
 **********************************************************************/
 char *get_cisml_program_name(CISML_T *cisml);
 
+/**********************************************************************
+  get_cisml_command_line
+
+  Gets the command_line member in a cisml object.
+**********************************************************************/
+char *get_cisml_command_line(CISML_T *cisml);
+
 /**********************************************************************
   get_cisml_pattern_file
 
@@ -108,7 +118,7 @@ void clear_cisml_pattern_pvalue_cutoff(CISML_T *cisml);
 
   Does a cisml object have a pattern_pvalue_cutoff?
 **********************************************************************/
-BOOLEAN_T has_cisml_pattern_pvalue_cutoff(CISML_T *cisml);
+bool has_cisml_pattern_pvalue_cutoff(CISML_T *cisml);
 
 /**********************************************************************
   get_cisml_pattern_pvalue_cutoff
@@ -140,7 +150,7 @@ void clear_cisml_sequence_pvalue_cutoff(CISML_T *cisml);
 
   Does a cisml object have a sequence_pvalue_cutoff?
 **********************************************************************/
-BOOLEAN_T has_cisml_sequence_pvalue_cutoff(CISML_T *cisml);
+bool has_cisml_sequence_pvalue_cutoff(CISML_T *cisml);
 
 /**********************************************************************
   get_cisml_sequence_pvalue_cutoff
@@ -150,6 +160,13 @@ BOOLEAN_T has_cisml_sequence_pvalue_cutoff(CISML_T *cisml);
 **********************************************************************/
 double get_cisml_sequence_pvalue_cutoff(CISML_T *cisml);
 
+/**********************************************************************
+  get_cisml_num_passing_cuttoff
+
+  Returns the number of matches passing the significance cuttoff
+**********************************************************************/
+int get_cisml_num_passing_cutoff(CISML_T *cisml);
+
 /**********************************************************************
   set_cisml_site_pvalue_cutoff
 
@@ -169,7 +186,7 @@ void clear_cisml_site_pvalue_cutoff(CISML_T *cisml);
 
   Does a cisml object have a pattern_pvalue_cutoff?
 **********************************************************************/
-BOOLEAN_T has_cisml_site_pvalue_cutoff(CISML_T *cisml);
+bool has_cisml_site_pvalue_cutoff(CISML_T *cisml);
 
 /**********************************************************************
   get_cisml_site_pvalue_cutoff
@@ -198,7 +215,7 @@ void clear_cisml_site_qvalue_cutoff(CISML_T *cisml);
 
   Does a cisml object have a site_qvalue_cutoff?
 **********************************************************************/
-BOOLEAN_T has_cisml_site_qvalue_cutoff(CISML_T *cisml);
+bool has_cisml_site_qvalue_cutoff(CISML_T *cisml);
 
 /**********************************************************************
   get_cisml_site_qvalue_cutoff
@@ -252,6 +269,13 @@ int get_cisml_num_patterns(CISML_T *cisml);
 **********************************************************************/
 PATTERN_T **get_cisml_patterns(CISML_T *cisml);
 
+/**********************************************************************
+  get_cisml_num_stored_matches
+
+  Gets the number of stored matches for all patterns in a cisml object.
+**********************************************************************/
+int get_cisml_num_stored_matches(CISML_T *cisml);
+
 /**********************************************************************
   add_cisml_multi_pattern
 
@@ -266,6 +290,30 @@ void add_cisml_multi_pattern(CISML_T *cisml, MULTI_PATTERN_T* multi_pattern);
 **********************************************************************/
 void add_cisml_pattern(CISML_T *cisml, PATTERN_T* pattern);
 
+/**********************************************************************
+  allocate_cisml_match_iterator
+
+  Constructor for an iterator used to traverse all matched elements
+  in in a cisml object. It stores the current index into the array of 
+  matched elements for each of the patterns in the CisML object.
+**********************************************************************/
+CISML_MATCH_IT_T *allocate_cisml_match_iterator(CISML_T *cisml);
+
+/**********************************************************************
+  free_cisml_match_iterator
+
+  Destructor for the CisML match iterator.
+**********************************************************************/
+void free_cisml_match_iterator(CISML_MATCH_IT_T *it);
+
+/**********************************************************************
+  cisml_match_iterator_next
+
+  Returns the next matched element from the CisML object.
+  Matches are returned in order of ascending p-value.
+***********************************************************************/
+MATCHED_ELEMENT_T *cisml_match_iterator_next(CISML_MATCH_IT_T *it);
+
 /**********************************************************************
   allocate_multi_pattern
 
@@ -292,7 +340,7 @@ void set_multi_pattern_score(MULTI_PATTERN_T *multi_pattern, double score);
 
   Does a multi_pattern object have a score?
 **********************************************************************/
-BOOLEAN_T has_multi_pattern_score(MULTI_PATTERN_T *multi_pattern);
+bool has_multi_pattern_score(MULTI_PATTERN_T *multi_pattern);
 
 /**********************************************************************
   clear_multi_pattern_score
@@ -320,7 +368,7 @@ void set_multi_pattern_pvalue(MULTI_PATTERN_T *multi_pattern, double pvalue);
 
   Does a multi_pattern object have a pvalue?
 **********************************************************************/
-BOOLEAN_T has_multi_pattern_pvalue(MULTI_PATTERN_T *multi_pattern);
+bool has_multi_pattern_pvalue(MULTI_PATTERN_T *multi_pattern);
 
 /**********************************************************************
   clear_multi_pattern_pvalue
@@ -343,6 +391,25 @@ double get_multi_pattern_pvalue(MULTI_PATTERN_T *multi_pattern);
 **********************************************************************/
 int get_multi_pattern_num_patterns(MULTI_PATTERN_T *multi_pattern);
 
+/**********************************************************************
+  get_multi_pattern_match
+
+  Sets the match member in a cisml multi-pattern object.
+**********************************************************************/
+MULTI_PATTERN_MATCH_T *get_multi_pattern_match(
+  MULTI_PATTERN_T *multi_pattern
+);
+
+/**********************************************************************
+  set_multi_pattern_match
+
+  Sets the pvalue member in a cisml pattern object.
+**********************************************************************/
+void set_multi_pattern_match(
+  MULTI_PATTERN_T *multi_pattern, 
+  MULTI_PATTERN_MATCH_T *match
+);
+
 /**********************************************************************
   get_multi_pattern_patterns
 
@@ -361,6 +428,63 @@ void add_multi_pattern_pattern(
   PATTERN_T* pattern
 );
 
+/*************************************************************************
+ * Calculate the q-values from the p-values of the multi-patterns
+ *************************************************************************/
+void multi_pattern_calculate_qvalues(
+  int num_multi_pattern,
+  MULTI_PATTERN_T **multi_patterns, 
+  ARRAY_T *sampled_pvalues
+);
+
+/**********************************************************************
+  allocate_multi_pattern_match
+
+  Constructor for the cisml multi_pattern_match data structure.
+**********************************************************************/
+MULTI_PATTERN_MATCH_T *allocate_multi_pattern_match(
+    char *seq_name,
+    char *seq,
+    int start,
+    int stop,
+    double pvalue
+);
+
+/**********************************************************************
+  free_multi_pattern_match
+
+  Destructor for the cisml multi_pattern_match data structure.
+**********************************************************************/
+void free_multi_pattern_match(MULTI_PATTERN_MATCH_T *match);
+
+/**********************************************************************
+  set_multi_pattern_match_evalue
+
+  Sets the evalue member in a cisml multi_pattern_match object.
+**********************************************************************/
+void set_multi_pattern_match_evalue(MULTI_PATTERN_MATCH_T *match, double evalue);
+
+/**********************************************************************
+  get_multi_pattern_match_pvalue
+
+  Gets the pvalue member from a cisml multi_pattern_match object.
+**********************************************************************/
+double get_multi_pattern_match_pvalue(MULTI_PATTERN_MATCH_T* match);
+
+/**********************************************************************
+  set_multi_pattern_match_qvalue
+
+  Sets the qvalue member in a cisml multi_pattern_match object.
+**********************************************************************/
+void set_multi_pattern_match_qvalue(MULTI_PATTERN_MATCH_T *match, double qvalue);
+
+/**********************************************************************
+  get_multi_pattern_match_qvalue
+
+  Gets the qvalue member from a cisml multi_pattern_match object.
+**********************************************************************/
+double get_multi_pattern_match_qvalue(MULTI_PATTERN_MATCH_T* match);
+
 /**********************************************************************
   allocate_pattern
 
@@ -396,11 +520,11 @@ void add_pattern_elements_to_scanned_seq(PATTERN_T *pattern);
   if it's greater the pvalues we've already thrown away, we refuse to
   add it. Also, check max_stored_matches. If we've
   hit the limit, purge the heap of the least significian matched elements.
-  Set to FALSE the Boolean that indicates that set of matched elements
+  Set to false the Boolean that indicates that set of matched elements
   is complete.
 
 **********************************************************************/
-BOOLEAN_T add_pattern_matched_element(
+bool add_pattern_matched_element(
   PATTERN_T *pattern,
   MATCHED_ELEMENT_T *element
 );
@@ -413,11 +537,19 @@ BOOLEAN_T add_pattern_matched_element(
   maximum number of elements, copying the elements from the existing heap,
   freeing the existing heap, and pointing the pattern to the new heap.
 
-  It fails if the new max is smaller then the current max_stored_matches.
+  It fails if the new max is smaller than the current max_stored_matches.
 
-  Returns TRUE if successful, FALSE otherwise.
+  Returns true if successful, false otherwise.
 **********************************************************************/
-BOOLEAN_T set_pattern_max_stored_matches(PATTERN_T *pattern, int max);
+bool set_pattern_max_stored_matches(PATTERN_T *pattern, int max);
+
+/**********************************************************************
+  set_pattern_max_pvalue_retained
+
+  Sets the maximum p-value of the matched elements retained by
+  the pattern.
+**********************************************************************/
+void set_pattern_max_pvalue_retained(PATTERN_T *pattern, double max_pvalue);
 
 /**********************************************************************
   get_pattern_max_stored_matches
@@ -435,22 +567,6 @@ int get_pattern_max_stored_matches(PATTERN_T *pattern);
 **********************************************************************/
 int get_pattern_num_stored_matches(PATTERN_T *pattern); 
 
-/**********************************************************************
-  get_pattern_has_all_pvalues
-
-  Returns the flag indicating whether or not all matched elements have 
-  been retained, or only those with the smallest p-values.
-**********************************************************************/
-BOOLEAN_T get_pattern_has_all_pvalues(PATTERN_T *pattern);
-
-/**********************************************************************
-  set_pattern_has_all_pvalues
-
-  Sets the flag indicating whether or not all matched elements have 
-  been retained, or only those with the smallest p-values.
-**********************************************************************/
-void set_pattern_has_all_pvalues(PATTERN_T *pattern, BOOLEAN_T has_all_pvalues);
-
 /**********************************************************************
   get_pattern_is_complete
 
@@ -459,7 +575,7 @@ void set_pattern_has_all_pvalues(PATTERN_T *pattern, BOOLEAN_T has_all_pvalues);
   longer available and all matched elements are stored in an array
   of matched element pointers, sorted by p-value.
 **********************************************************************/
-BOOLEAN_T get_pattern_is_complete(PATTERN_T *pattern);
+bool get_pattern_is_complete(PATTERN_T *pattern);
 
 /**********************************************************************
   set_pattern_is_complete
@@ -513,7 +629,7 @@ void clear_pattern_pvalue(PATTERN_T *pattern);
 
   Does a pattern object have a pvalue?
 **********************************************************************/
-BOOLEAN_T has_pattern_pvalue(PATTERN_T *pattern);
+bool has_pattern_pvalue(PATTERN_T *pattern);
 
 /**********************************************************************
   get_pattern_pvalue
@@ -527,7 +643,7 @@ double get_pattern_pvalue(PATTERN_T* pattern);
 
   Does the matched-elements for the pattern have qvalues available?
 **********************************************************************/
-BOOLEAN_T has_pattern_qvalues(PATTERN_T *pattern);
+bool has_pattern_qvalues(PATTERN_T *pattern);
 
 /**********************************************************************
   set_pattern_score
@@ -548,7 +664,7 @@ void clear_pattern_pvalue(PATTERN_T *pattern);
 
   Does a pattern object have a score?
 **********************************************************************/
-BOOLEAN_T has_pattern_score(PATTERN_T *pattern);
+bool has_pattern_score(PATTERN_T *pattern);
 
 /**********************************************************************
   get_pattern_score
@@ -595,19 +711,44 @@ char *get_pattern_lsid(PATTERN_T* pattern);
 int get_pattern_num_scanned_sequences(PATTERN_T *pattern);
 
 /**********************************************************************
-  get_pattern_num_scanned_elements
+  get_pattern_num_scanned_positions
 
   Gets the number of sites scanned with a cisml pattern object.
 **********************************************************************/
-int get_pattern_num_scanned_elements(PATTERN_T *pattern);
+long get_pattern_num_scanned_positions(PATTERN_T *pattern);
 
 /**********************************************************************
-  get_pattern_num_matched_elements
+  get_pattern_num_stored_matches
 
-  Gets the total number of matched_element objects contained in a cisml
+  Gets the total number of matched element objects stored in a cisml
   pattern object.
 **********************************************************************/
-int get_pattern_num_matched_elements(PATTERN_T *pattern);
+int get_pattern_num_stored_matches(PATTERN_T *pattern);
+
+/**********************************************************************
+  get_pattern_stored_matches
+
+  Gets the array of matched element objects stored in a cisml
+  pattern object.
+**********************************************************************/
+MATCHED_ELEMENT_T **get_pattern_stored_matches(PATTERN_T *pattern);
+
+/**********************************************************************
+  get_pattern_matched_elements
+
+  Gets the array of matched_element objects contained in a cisml
+  pattern object.
+**********************************************************************/
+MATCHED_ELEMENT_T **get_pattern_matched_elements(PATTERN_T *pattern);
+
+/*************************************************************************
+ * Calculate the q-values for each matched-element in this pattern
+ * from the p-values.
+ *************************************************************************/
+void pattern_calculate_qvalues(
+  PATTERN_T *pattern,
+  ARRAY_T *sampled_pvalues
+);
 
 /**********************************************************************
   get_pattern_scanned_sequences
@@ -691,7 +832,7 @@ void clear_scanned_sequence_pvalue(SCANNED_SEQUENCE_T *scanned_sequence);
 
   Does a scanned_sequence object have a pvalue?
 **********************************************************************/
-BOOLEAN_T has_scanned_sequence_pvalue(SCANNED_SEQUENCE_T *scanned_sequence);
+bool has_scanned_sequence_pvalue(SCANNED_SEQUENCE_T *scanned_sequence);
 
 /**********************************************************************
   get_scanned_sequence_pvalue
@@ -700,37 +841,6 @@ BOOLEAN_T has_scanned_sequence_pvalue(SCANNED_SEQUENCE_T *scanned_sequence);
 **********************************************************************/
 double get_scanned_sequence_pvalue(SCANNED_SEQUENCE_T* scanned_sequence);
 
-/**********************************************************************
-  clear_scanned_sequence_raw_seq
-
-  Sets the raw_seq member in a cisml scanned_sequence object to null.
-**********************************************************************/
-void clear_scanned_sequence_raw_seq(SCANNED_SEQUENCE_T *scanned_sequence);
-
-/**********************************************************************
-  has_scanned_sequence_raw_seq
-
-  Does a scanned_sequence object have a raw seq?
-**********************************************************************/
-BOOLEAN_T has_scanned_sequence_raw_seq(SCANNED_SEQUENCE_T *scanned_sequence);
-
-/**********************************************************************
-  get_scanned_sequence_raw_ seq
-
-  Retruns a copy of the raw_seq member in a cisml scanned_sequence object.
-**********************************************************************/
-char* get_scanned_sequence_raw_seq(SCANNED_SEQUENCE_T *scanned_sequence);
-
-/**********************************************************************
-  set_scanned_sequence_raw_ seq
-
-  Sets the raw_seq member in a cisml scanned_sequence object.
-**********************************************************************/
-void set_scanned_sequence_raw_seq(
-  SCANNED_SEQUENCE_T *scanned_sequence,
-  char *seq 
-);
-
 /**********************************************************************
   set_scanned_sequence_score
 
@@ -748,6 +858,13 @@ void set_scanned_sequence_score(
 **********************************************************************/
 void clear_scanned_sequence_score(SCANNED_SEQUENCE_T *scanned_sequence);
 
+/**********************************************************************
+  has_scanned_sequence_score
+
+  Does a scanned_sequence object have a score?
+**********************************************************************/
+bool has_scanned_sequence_score(SCANNED_SEQUENCE_T *scanned_sequence);
+
 /**********************************************************************
   get_scanned_sequence_score
 
@@ -777,7 +894,7 @@ void clear_scanned_sequence_length(SCANNED_SEQUENCE_T *scanned_sequence);
 
   Does a scanned_sequence object have a length?
 **********************************************************************/
-BOOLEAN_T has_scanned_sequence_length(SCANNED_SEQUENCE_T *scanned_sequence);
+bool has_scanned_sequence_length(SCANNED_SEQUENCE_T *scanned_sequence);
 
 /**********************************************************************
   get_scanned_sequence_length
@@ -830,11 +947,11 @@ void add_scanned_sequence_matched_element(
 );
 
 /**********************************************************************
-  add_scanned_sequence_scanned_element
+  add_scanned_sequence_scanned_position
 
   Increments the count of scanned elements in a scanned_sequence.
 **********************************************************************/
-void add_scanned_sequence_scanned_element( SCANNED_SEQUENCE_T *sequence);
+void add_scanned_sequence_scanned_position(SCANNED_SEQUENCE_T *sequence);
 
 /**********************************************************************
   get_scanned_sequence_num_matched_elements
@@ -845,12 +962,12 @@ void add_scanned_sequence_scanned_element( SCANNED_SEQUENCE_T *sequence);
 int get_scanned_sequence_num_matched_elements(SCANNED_SEQUENCE_T *sequence);
 
 /**********************************************************************
-  get_scanned_sequence_num_scanned_elements
+  get_scanned_sequence_num_scanned_positions
 
   Gets the number of positions in the scanned_sequence where we
   have scanned for a matched_element.
 **********************************************************************/
-int get_scanned_sequence_num_scanned_elements(SCANNED_SEQUENCE_T *sequence);
+long get_scanned_sequence_num_scanned_positions(SCANNED_SEQUENCE_T *sequence);
 
 /**********************************************************************
   get_scanned_sequence_num_matched_elements
@@ -860,6 +977,14 @@ int get_scanned_sequence_num_scanned_elements(SCANNED_SEQUENCE_T *sequence);
 **********************************************************************/
 int get_scanned_sequence_num_matched_elements(SCANNED_SEQUENCE_T *sequence);
 
+/**********************************************************************
+  get_scanned_sequence_num_scanned_positions
+
+  Gets the number of positions in the scanned_sequence where we
+  have scanned for a matched_element.
+**********************************************************************/
+long get_scanned_sequence_num_scanned_positions(SCANNED_SEQUENCE_T *sequence);
+
 /**********************************************************************
   get_scanned_sequences_elements
 
@@ -909,7 +1034,7 @@ MATCHED_ELEMENT_T *allocate_matched_element(
 MATCHED_ELEMENT_T *allocate_matched_element_without_inversion(
   int start, 
   int stop, 
-  char *seq,
+  const char *seq,
   SCANNED_SEQUENCE_T *parent
 );
 
@@ -960,6 +1085,13 @@ void set_matched_element_start(MATCHED_ELEMENT_T* matched_element, int newstart)
 **********************************************************************/
 int get_matched_element_stop(MATCHED_ELEMENT_T* matched_element);
 
+/**********************************************************************
+  set_matched_element_stop
+
+  sets the stop member from a cisml matched_element object.
+**********************************************************************/
+void set_matched_element_stop(MATCHED_ELEMENT_T* element, int newstop);
+
 /**********************************************************************
   set_matched_element_score
 
@@ -970,19 +1102,12 @@ void set_matched_element_score(
   double score
 );
 
-/**********************************************************************
-  clear_matched_element_score
-
-  Sets the score member in a cisml matched_element object to null.
-**********************************************************************/
-void clear_matched_element_score(MATCHED_ELEMENT_T *element);
-
 /**********************************************************************
   has_matched_element_score
 
   Does a matched_element object have a score?
 **********************************************************************/
-BOOLEAN_T has_matched_element_score(MATCHED_ELEMENT_T *element);
+bool has_matched_element_score(MATCHED_ELEMENT_T *element);
 
 /**********************************************************************
   get_matched_element_score
@@ -1001,19 +1126,12 @@ void set_matched_element_pvalue(
   double pvalue
 );
 
-/**********************************************************************
-  clear_matched_element_pvalue
-
-  Sets the pvalue member in a cisml matched_element object to null.
-**********************************************************************/
-void clear_matched_element_pvalue(MATCHED_ELEMENT_T *element);
-
 /**********************************************************************
   has_matched_element_pvalue
 
   Does a matched_element object have a pvalue?
 **********************************************************************/
-BOOLEAN_T has_matched_element_pvalue(MATCHED_ELEMENT_T *element);
+bool has_matched_element_pvalue(MATCHED_ELEMENT_T *element);
 
 /**********************************************************************
   get_matched_element_pvalue
@@ -1022,6 +1140,20 @@ BOOLEAN_T has_matched_element_pvalue(MATCHED_ELEMENT_T *element);
 **********************************************************************/
 double get_matched_element_pvalue(MATCHED_ELEMENT_T* element);
 
+/**********************************************************************
+  has_matched_element_qvalue
+
+  Does a matched_element object have a qvalue?
+**********************************************************************/
+bool has_matched_element_qvalue(MATCHED_ELEMENT_T *element);
+
+/**********************************************************************
+  get_matched_element_qvalue
+
+  Gets the qvalue member from a cisml matched_element object.
+**********************************************************************/
+double get_matched_element_qvalue(MATCHED_ELEMENT_T* element);
+
 /**********************************************************************
   set_matched_element_clusterid
 
@@ -1044,10 +1176,38 @@ char *get_matched_element_clusterid(MATCHED_ELEMENT_T* element);
   get_matched_element_sequence
 
   Gets the sequence member from a cisml matched_element object.
+  Caller should not free the string.
   May return NULL.
 **********************************************************************/
-char *get_matched_element_sequence(MATCHED_ELEMENT_T* element);
+const char *get_matched_element_sequence(MATCHED_ELEMENT_T* element);
 
+/**********************************************************************
+  set_matched_element_strand
+
+  Sets the strand member in a cisml matched_element object.
+**********************************************************************/
+void set_matched_element_strand(MATCHED_ELEMENT_T* element, char strand);
+
+/**********************************************************************
+  get_matched_element_strand
+
+  Gets the strand member in a cisml matched_element object.
+**********************************************************************/
+char get_matched_element_strand(MATCHED_ELEMENT_T* element);
+
+/**********************************************************************
+  get_matched_element_scanned_seq
+
+  Returns the parent_scanned_sequence member in a matched_element object.
+**********************************************************************/
+SCANNED_SEQUENCE_T *get_matched_element_scanned_seq(MATCHED_ELEMENT_T* element);
+
+/**********************************************************************
+  set_matched_element_sequence
+
+  Sets the sequence member in a cisml matched_element object.
+**********************************************************************/
+void set_matched_element_sequence(MATCHED_ELEMENT_T* element, char *seq);
 
 /*************************************************************************
  * sort_matched_elements
@@ -1058,20 +1218,12 @@ char *get_matched_element_sequence(MATCHED_ELEMENT_T* element);
  * JH - I opened this up to make it accessible outside of the CISML code
  * because I needed the matched elements sorted for the BLS implementation
  *************************************************************************/
-
 void sort_matched_elements(
-  BOOLEAN_T sort_by_pvalue,
+  bool sort_by_pvalue,
   int num_elements,
   MATCHED_ELEMENT_T **elements
 );
 
-/**********************************************************************
-  set_matched_element_sequence
-
-  Sets the sequence member in a cisml matched_element object.
-**********************************************************************/
-void set_matched_element_sequence(MATCHED_ELEMENT_T* element, char *seq);
-
 /**********************************************************************
   print_cisml_start
 
@@ -1079,10 +1231,10 @@ void set_matched_element_sequence(MATCHED_ELEMENT_T* element, char *seq);
 **********************************************************************/
 void print_cisml_start(
   FILE* out, 
-  CISML_T *cisml, 
-  BOOLEAN_T print_header,
+  char *program_name, 
+  bool print_header,
   const char *stylesheet, 
-  BOOLEAN_T print_namespace
+  bool print_namespace
 );
 
 /**********************************************************************
@@ -1107,17 +1259,11 @@ void print_cisml_parameters(FILE *out, CISML_T *cisml);
 void print_cisml(
 		FILE* out,                /* The target file IN */
 		CISML_T *cisml,           /* The cisml data structure to save IN */
-		BOOLEAN_T print_header,   /* FLAG indicates if the header should be printed IN */
+		bool print_header,   /* FLAG indicates if the header should be printed IN */
 		const char *stylesheet,   /* the stylesheet file IN */
-		BOOLEAN_T print_namespace /* FLAG indicates if the namespace should be printed IN */
+		bool print_namespace /* FLAG indicates if the namespace should be printed IN */
 );
 
-/*************************************************************************
- * Calculate the q-values for each matched-element in this pattern
- * from the p-value.
- *************************************************************************/
-void pattern_calculate_qvalues(PATTERN_T *pattern);
-
 /**********************************************************************
   print_cisml_start_pattern
 
@@ -1180,84 +1326,31 @@ void print_cisml_scanned_sequence_end(FILE *out);
  *   print_namespace will determine whether or not the standard name
  *   				space will be written
  *********************************************************************/
-void print_full_results(
+bool print_full_results(
   CISML_T *cisml,
   char *output_dirname,
   char *xml_filename,
   char *html_filename,
   char *text_filename,
   char *gff_filename,
-  BOOLEAN_T allow_clobber,
-  BOOLEAN_T print_namespace
+  bool allow_clobber,
+  bool print_namespace
 );
 
 /**********************************************************************
  * This function saves the CisML results as a set of files in a
  * directory. The files include:
  *   fimo.xml which uses the CisML schema
- *   fimo.txt which uses plain text with tab-delimited columns
+ *   fimo.tsv which uses plain text with tab-delimited columns
  *   fimo.gff which uses the gff file format.
  *********************************************************************/
-void print_cisml_as_text(CISML_T *cisml);
+bool print_cisml_as_text(CISML_T *cisml);
 
 /**********************************************************************
   read_cisml
 
   Reads in a CisML XML file and create the cisml_t data structure
 **********************************************************************/
-CISML_T* read_cisml(
-	char* 		cisml_filename // name of the cisml file
-);
-
-/**********************************************************************
-  read_multiple_patterns_from_cisml
-
-  reads a multi-pattern-scan from a CisML xml file
-**********************************************************************/
-void read_multiple_patterns_from_cisml(
-		xmlXPathContextPtr 	xpath_ctxt,    	// cisml XPath context.
-		int* 				num_patterns,	// cisml number of pattern in the file
-		CISML_T* 			cisml			// the retrieved cisml struct
-);
-
-/**********************************************************************
-  read_patterns_from_cisml
-
-  reads the patterns from a CisML xml file
-**********************************************************************/
-void read_patterns_from_cisml(
-		xmlXPathContextPtr 	xpath_ctxt,    	// cisml XPath context.
-		int* 				num_patterns,	// cisml number of pattern in the file
-		CISML_T* 			cisml			// the retrieved cisml struct
-);
-
-/**********************************************************************
-  get_pattern_from_xmlnode
-
-  returns a pattern from a specified xml Node
-**********************************************************************/
-PATTERN_T* get_pattern_from_xmlnode(xmlNodePtr currPatternNode);
-
-/**********************************************************************
-  read_scanned_sequences_from_cisml
-
-  Reads in the sequences for a pattern
-**********************************************************************/
-void read_scanned_sequences_from_cisml(
-  CISML_T* 				cisml,			// the retrieved cisml struct
-  PATTERN_T*			pattern, 		// the pattern of interest
-  xmlNode*				first_sequence	// first child of the pattern node
-);
-
-/**********************************************************************
-  read_matched_elements_from_cisml
-
-  Reads in sequences matches from a cisml file
-**********************************************************************/
-void  read_matched_elements_from_cisml(
-  CISML_T* 				cisml,			// the retrieved cisml struct
-  SCANNED_SEQUENCE_T*	scanned_seq,	// the pattern of interest
-  xmlNode*				firstmatch		// first child of the pattern node
-);
+CISML_T* read_cisml(const char* cisml_filename);
 
 #endif
