@@ -1710,7 +1710,7 @@ void print_meme_file_xml(
   char* xml_filename              // full path to output file for xml IN 
 )
 {
-  FILE* outfile = fopen(xml_filename, "w"); //FIXME CEG check for errors
+  FILE* outfile = fopen(xml_filename, "w"); //TODO CEG check for errors
   print_meme_header_xml(outfile);
 
   fputs("<MEME version=\"" VERSION "\" release=\"" ARCHIVE_DATE "\">\n", outfile);
@@ -1854,9 +1854,7 @@ static void print_meme_model_xml(
   FILE* outfile          // output file IN 
 ) {
   char evt_string[20];
-  STR_T *b;
- 
-  b = str_create(10);
+  STR_T *b = str_create(10);
 
   if (dataset->evt == BIG) {
     strcpy(evt_string, "inf");
