@@ -29,7 +29,7 @@
 #define PROTEIN_PLIB "prior30.plib"
 
 #define ROUNDERROR (1E-12)
-//FIXME: how large should these limits be?
+//TODO: how large should these limits be?
 #define DEMIN 2
 #define NZMIN 2
 
@@ -339,7 +339,7 @@ void init_meme(
       // Create the name of the text output file 
       // "<dir>/text_filename/" and open it for writing
       char *path = make_path_to_file(*output_dirname, text_filename);
-      *text_output = fopen(path, "w"); //FIXME CEG check for errors
+      *text_output = fopen(path, "w"); //TODO CEG check for errors
       myfree(path);
     }
   }
