@@ -197,7 +197,7 @@ char* MAST_DTD =
 #define STRINGIZE(s) STRINGIZE2(s)
 
  // default e-thresh 
-#define EXPECT 10
+#define EXPECT 10.0
 
 #define MOTIF_THRESH_DEFAULT 1e-4
 
@@ -2194,7 +2194,7 @@ int main(int argc, char *argv[])
     double evalue = st->Pvalue * nseqs;
     if (evalue <= options.e_max) break;
   }
-  if (++i < arraylst_size(hits)) {//TODO FIXME check if the hits are freed somewhere
+  if (++i < arraylst_size(hits)) {//TODO check if the hits are freed somewhere
     arraylst_remove_range(i, arraylst_size(hits) - i, NULL, hits);
   }
 
