@@ -224,15 +224,15 @@ char* regexmotif_to_string(REGEX_MOTIF_T* regexmotif,
   for (i = 0; i < options->width; ++i) {
     int idx_no_center = (i < width_no_center / 2) ? i : i - 1;
     if (i == options->width / 2) {
-      strncat(result, "_", 1);
-      strncat(result, mod_info->mod_name, strlen(mod_info->mod_name));
-      strncat(result, "_", 1);
+      strcat(result, "_");
+      strcat(result, mod_info->mod_name);
+      strcat(result, "_");
     } else if (!regexmotif->conserved[idx_no_center]) {
-      strncat(result, "x", 1);
+      strcat(result, "x");
     } else {
       int residues_length = get_residues_length(regexmotif, summary, idx_no_center);
       if (residues_length > 1) {
-        strncat(result, "[", 1);
+        strcat(result, "[");
       }
       for (j = 0; j < strlen(alph_letters); ++j) {
         if (get_matrix_cell(idx_no_center, j, regexmotif->residues) == 1.0) {
@@ -240,7 +240,7 @@ char* regexmotif_to_string(REGEX_MOTIF_T* regexmotif,
         }
       }
       if (residues_length > 1) {
-        strncat(result, "]", 1);
+        strcat(result, "]");
       }
     }
   }
