@@ -198,8 +198,8 @@ void momo_print_motifs(
 
       char* motif_png = mm_malloc(strlen(motif_name) + 5);
       motif_png[0] = '\0';
-      strncat(motif_png, motif_name, strlen(motif_name));
-      strncat(motif_png, ".png", 4);
+      strcat(motif_png, motif_name);
+      strcat(motif_png, ".png");
       // Replace [] wight bd because square brackets are significant in URI.
       int n; 
       for (n=0; n<strlen(motif_name); n++) {
@@ -212,8 +212,8 @@ void momo_print_motifs(
       
       char* logo_file = mm_malloc(strlen(options.output_dirname) + strlen(motif_name) + 2);
       logo_file[0] = '\0';
-      strncat(logo_file, options.output_dirname, strlen(options.output_dirname));
-      strncat(logo_file, "/", 1);
+      strcat(logo_file, options.output_dirname);
+      strcat(logo_file, "/");
       strncat(logo_file, motif_png, strlen(motif_name));
       CL_create1(currmotif, false, false, "MoMo", logo_file, false, true);
       
