@@ -1036,8 +1036,8 @@ void read_motif_alphabets(ARRAYLST_T* motif_sources, bool xalph, ALPH_T** alph)
 		  alph_name(db_alph), alph_name(*alph));
                 break;
               case -1:
-                fprintf(stderr, "Warning: the alphabet expansion from %s to %s"
-		  " requires changing complementing rules.", 
+                fprintf(stderr, "Warning: the alphabet expansion from '%s' to '%s'"
+		  " requires changing complementation rules.\n", 
 		  alph_name(db_alph), alph_name(*alph));
                 break;
             }
