@@ -18,8 +18,8 @@
 #define MAX_ENTRY 100
 
 struct scored_site_t {
-  char   seq_name[MAX_ENTRY];
-  char   feature_type[MAX_ENTRY];
+  char   seq_name[MAX_ENTRY+1];
+  char   feature_type[MAX_ENTRY+1];
   int    start;
   int    end;
   int    seq_length;
@@ -27,8 +27,8 @@ struct scored_site_t {
   double pvalue;
   double qvalue;
   char   strand;
-  char   motif_id[MAX_ENTRY];
-  char   window_seq[MAX_ENTRY];
+  char   motif_id[MAX_ENTRY+1];
+  char   window_seq[MAX_ENTRY+1];
 };
 
 struct scored_sites_t {
