@@ -15,7 +15,6 @@
 #include "clustalw-io.h"
 #include "evomodel.h"
 #include "fasta-io.h"
-#include "meme-io.h"
 #include "mhmm-state.h"
 #include "motif.h"
 #include "motiph-scoring.h"
@@ -24,9 +23,6 @@
 #include "simple-getopt.h"
 #include "tree.h"
 
-// Nucleotide alphabest order as in motif.h
-extern char alphabet[];
-
 char* program_name = NULL;
 VERBOSE_T verbosity = NORMAL_VERBOSE;
 
@@ -41,15 +37,15 @@ static ARRAY_T* read_bg_from_alignment(
  *************************************************************************/
 int main(int argc, char *argv[]) {
 
-  BOOLEAN_T compute_pvalues = TRUE;
-  BOOLEAN_T compute_qvalues = TRUE;
-  BOOLEAN_T use_file_list = FALSE;
-  BOOLEAN_T allow_clobber = TRUE;
-  BOOLEAN_T text_only = FALSE;
-  BOOLEAN_T output_pthresh_set = FALSE;
-  BOOLEAN_T output_qthresh_set = FALSE;
+  bool compute_pvalues = true;
+  bool compute_qvalues = true;
+  bool use_file_list = false;
+  bool allow_clobber = true;
+  bool text_only = false;
+  bool output_pthresh_set = false;
+  bool output_qthresh_set = false;
   int ref_seq_index = 0;  // Which sequence provides reference coords.
-  //FIXME: Eventually, above should be accessible via a command-line switch.
+  //TODO: Eventually, above should be accessible via a command-line switch.
   char *output_dirname = "shadow_out";
   char* bg_filename = NULL;
   int max_stored_scores = 100000;
@@ -70,9 +66,10 @@ int main(int argc, char *argv[]) {
    **********************************************/
 
   // Define command line options.
-  const int num_options = 18;
+  const int num_options = 19;
   cmdoption const shadow_options[] = {
     {"bg", REQUIRED_VALUE},
+    {"bfile", REQUIRED_VALUE},
     {"bgfile", REQUIRED_VALUE},
     {"fg", REQUIRED_VALUE},
     {"gap", REQUIRED_VALUE},
@@ -96,7 +93,7 @@ int main(int argc, char *argv[]) {
 
   // Define the usage message.
   char usage[1000] = "";
-  strcat(usage, "USAGE: shadow [options] <alignment> <tree>\n");
+  strcat(usage, "Usage: shadow [options] <alignment> <tree>\n");
   strcat(usage, "\n");
   strcat(usage, "   Options:\n");
 
@@ -113,7 +110,7 @@ int main(int argc, char *argv[]) {
   strcat(usage, "     --transition-transversion <float> (default=0.5)\n");
 
   // Miscellaneous parameters
-  strcat(usage, "     --bgfile <background> (default from alignment)\n");
+  strcat(usage, "     --bfile <background> (default from alignment)\n");
   strcat(usage, "     --max-stored-scores (Default 100,000)\n");
   strcat(usage, "     --no-pvalue (default false)\n");
   strcat(usage, "     --no-qvalue (default false)\n");
@@ -130,7 +127,7 @@ int main(int argc, char *argv[]) {
     die("Error processing command line options: option name too long.\n");
   }
 
-  while (TRUE) {
+  while (true) {
     int c = 0;
     char* option_name = NULL;
     char* option_value = NULL;
@@ -148,7 +145,7 @@ int main(int argc, char *argv[]) {
     if (strcmp(option_name, "bg") == 0){
       bg_rate = atof(option_value);
     }
-    else if (strcmp(option_name, "bgfile") == 0){
+    else if (strcmp(option_name, "bgfile") == 0 || strcmp(option_name, "bgfile") == 0){
       bg_filename = option_value;
     }
     else if (strcmp(option_name, "fg") == 0){
@@ -175,7 +172,7 @@ int main(int argc, char *argv[]) {
       gap_cost = atof(option_value);
     }
     else if (strcmp(option_name, "list") == 0){
-      use_file_list = TRUE;
+      use_file_list = true;
     }
     else if (strcmp(option_name, "model") == 0) {
       if (strcmp(option_value, "jc") == 0) {
@@ -199,37 +196,37 @@ int main(int argc, char *argv[]) {
       }
     }
     else if (strcmp(option_name, "no-pvalue") == 0){
-      compute_pvalues = FALSE;
-      compute_qvalues = FALSE;
+      compute_pvalues = false;
+      compute_qvalues = false;
     }
     else if (strcmp(option_name, "no-qvalue") == 0){
-      compute_qvalues = FALSE;
+      compute_qvalues = false;
     }
     else if (strcmp(option_name, "o") == 0){
       // Set output directory with no clobber
       output_dirname = option_value;
-      allow_clobber = FALSE;
+      allow_clobber = false;
     }
     else if (strcmp(option_name, "oc") == 0){
       // Set output directory with clobber
       output_dirname = option_value;
-      allow_clobber = TRUE;
+      allow_clobber = true;
     }
     else if (strcmp(option_name, "output-pthresh") == 0){
       output_pthresh = atof(option_value);
       output_qthresh = 1.0;
-      output_pthresh_set = TRUE;
+      output_pthresh_set = true;
     }
     else if (strcmp(option_name, "output-qthresh") == 0){
       output_qthresh = atof(option_value);
       output_pthresh = 1.0;
-      output_qthresh_set = TRUE;
+      output_qthresh_set = true;
     }
     else if (strcmp(option_name, "pur-pyr") == 0){
         purine_pyrimidine = atof(option_value);
     }
     else if (strcmp(option_name, "text") == 0){
-      text_only = TRUE;
+      text_only = true;
     }
     else if (strcmp(option_name, "transition-transervsion") == 0){
         transition_transversion = atof(option_value);
@@ -240,25 +237,23 @@ int main(int argc, char *argv[]) {
   }
 
   // Check that pvalue and qvalue options are consistent
-  if (compute_pvalues == FALSE
-    && (output_pthresh_set == TRUE || output_qthresh_set == TRUE)) {
+  if (compute_pvalues == false
+    && (output_pthresh_set == true || output_qthresh_set == true)) {
     die(
       "The --no-pvalue option cannot be used with the\n"
       " --output-pthresh or the --output-qthresh options"
     );
   }
-  if (compute_qvalues == FALSE && output_qthresh_set == TRUE) {
+  if (compute_qvalues == false && output_qthresh_set == true) {
     die("The --no-qvalue option cannot be used with the --output-qthresh options");
   }
 
   // Must have tree and alignment file names
   if (argc != option_index + 2) {
-    fprintf(stderr, usage);
+    fprintf(stderr, "%s", usage);
     exit(EXIT_FAILURE);
   }
 
-  set_alphabet(verbosity, "ACGT");
-
   /****************************************************
    * Read the names of the alignment files.
    ****************************************************/
@@ -333,8 +328,8 @@ int main(int argc, char *argv[]) {
     char* filename = get_nth_string(file_index, filenames);
     ALIGNMENT_T* alignment = read_alignment_from_file(
       filename,
-      TRUE, // Sort by species name
-      FALSE, // Remove gaps
+      true, // Sort by species name
+      false, // Remove gaps
       &current_ref_seq_index // update index to reference sequence
     );
     if (verbosity >= NORMAL_VERBOSE) {
@@ -348,7 +343,7 @@ int main(int argc, char *argv[]) {
     }
 
     // Trim the tree, eliminating species not in this alignment.
-    TREE_T* trimmed_tree = trim_tree(TRUE, tree, alignment_species);
+    TREE_T* trimmed_tree = trim_tree(true, tree, alignment_species);
 
     // Check that at least one species was found.
     if (trimmed_tree == NULL) {
@@ -390,7 +385,7 @@ int main(int argc, char *argv[]) {
 			transition_transversion,
 			purine_pyrimidine,
 			bg_freqs,
-			FALSE
+			false
     );
 
     // Build the foreground model using the equilibrium frequencies
@@ -401,7 +396,7 @@ int main(int argc, char *argv[]) {
 			transition_transversion,
 			purine_pyrimidine,
 			bg_freqs,
-			FALSE
+			false
     );
     if (verbosity >= HIGH_VERBOSE) {
       fprintf(stderr, "Created foreground and background model.\n");
@@ -416,6 +411,7 @@ int main(int argc, char *argv[]) {
     if (compute_pvalues) {
       // Build PSSM for motiph
       MATRIX_T* pssm_matrix = build_alignment_pssm_matrix(
+        DNA_ALPH,
         alignment_species,
         window_size + 1,
         models,
@@ -434,6 +430,7 @@ int main(int argc, char *argv[]) {
 
       // Build tables to translate log-odds scores to p-values
       PSSM_T* pssm = build_matrix_pssm(
+        DNA_ALPH,
 	pssm_matrix,
 	alignment_col_probs,
 	PSSM_RANGE 
@@ -443,6 +440,7 @@ int main(int argc, char *argv[]) {
       }
 
       score_sequence_in_alignment(
+        DNA_ALPH,
         current_ref_seq_index,
         trimmed_alignment,
         "shadow", // motif_id
@@ -464,9 +462,10 @@ int main(int argc, char *argv[]) {
     // No pvalues; don't precompute and store scores.  Much slower.
     else {
       score_sequence_in_alignment(
+                                DNA_ALPH,
 				ref_seq_index,
 				trimmed_alignment,
-				"shadow", // motif ID (FIXME?)
+				"shadow", // motif ID (TODO?)
 				trimmed_tree,
 				1, // Window size
 				models, // 0 = background, 1 = foreground
@@ -481,7 +480,7 @@ int main(int argc, char *argv[]) {
     }
 
     free_string_list(alignment_species);
-    free_tree(TRUE, trimmed_tree);
+    free_tree(true, trimmed_tree);
     free_alignment(trimmed_alignment);
     if (models != NULL) {
       int model_index;
@@ -496,7 +495,7 @@ int main(int argc, char *argv[]) {
 
   // Compute q-values, if requested.
   if (compute_pvalues && compute_qvalues) {
-    pattern_calculate_qvalues(pattern);
+    pattern_calculate_qvalues(pattern, NULL);
   }
 
   // Write out results
@@ -512,12 +511,12 @@ int main(int argc, char *argv[]) {
       "shadow.txt",
       "shadow.gff",
       allow_clobber,
-      TRUE
+      true
     );
   }
 
   free_array(bg_freqs);
-  free_tree(TRUE, tree);
+  free_tree(true, tree);
   free_string_list(filenames);
   free_cisml(cisml);
 
@@ -530,7 +529,7 @@ int main(int argc, char *argv[]) {
 ARRAY_T* read_bg_from_file(char* bg_filename) {
 
   FILE* bg_file = NULL;
-  if (open_file(bg_filename, "r", TRUE, "background",
+  if (open_file(bg_filename, "r", true, "background",
                 "background nucleotide frequencies", &bg_file) == 0) {
     die("Couldn't open the file %s.\n", bg_filename);
   }
@@ -544,7 +543,7 @@ ARRAY_T* read_bg_from_file(char* bg_filename) {
   float c_freq = 0.0;
   float t_freq = 0.0;
   float g_freq = 0.0;
-  while (TRUE) {
+  while (true) {
     if (fgets(line, 80, bg_file) == NULL) {
       break;
     }
@@ -580,17 +579,12 @@ ARRAY_T* read_bg_from_file(char* bg_filename) {
     die("Can't read background frequences from file %s.\n", bg_filename);
   }
 
-  int alph_size = get_alph_size(ALPH_SIZE);
-  ARRAY_T* bg_freqs = allocate_array(alph_size);
+  ARRAY_T* bg_freqs = allocate_array(alph_size(DNA_ALPH, ALPH_SIZE));
   int index = 0;
-  index = alphabet_index('A', alphabet);
-  set_array_item(index, a_freq, bg_freqs);
-  index = alphabet_index('C', alphabet);
-  set_array_item(index, c_freq, bg_freqs);
-  index = alphabet_index('G', alphabet);
-  set_array_item(index, g_freq, bg_freqs);
-  index = alphabet_index('T', alphabet);
-  set_array_item(index, t_freq, bg_freqs);
+  set_array_item(alph_index(DNA_ALPH, 'A'), a_freq, bg_freqs);
+  set_array_item(alph_index(DNA_ALPH, 'C'), c_freq, bg_freqs);
+  set_array_item(alph_index(DNA_ALPH, 'G'), g_freq, bg_freqs);
+  set_array_item(alph_index(DNA_ALPH, 'T'), t_freq, bg_freqs);
 
   return bg_freqs;
 }
@@ -607,21 +601,21 @@ static ARRAY_T* read_bg_from_alignment(
   ARRAY_T* bg_freqs;
   if (model_type == SINGLE_MODEL) {
     SEQ_T* seq = get_alignment_sequence(0, alignment);
-    bg_freqs = get_sequence_freqs(seq);
+    bg_freqs = get_sequence_freqs(seq, DNA_ALPH);
     free_seq(seq);
     if (verbosity >= NORMAL_VERBOSE) {
       fprintf(stderr, "Computing background frequencies from %s.\n",
               get_seq_name(seq));
     }
   } else {
-    bg_freqs = get_alignment_freqs(alignment);
+    bg_freqs = get_alignment_freqs(DNA_ALPH, alignment);
     if (verbosity >= NORMAL_VERBOSE) {
       fprintf(stderr, "Computing background frequencies from alignment.\n");
     }
   }
   if (verbosity >= NORMAL_VERBOSE) {
     fprintf(stderr, "Background frequencies: ");
-    print_array(bg_freqs, 6, 3, TRUE, stderr);
+    print_array(bg_freqs, 6, 3, true, stderr);
   }
 
   return bg_freqs;
