@@ -16,8 +16,9 @@
 #endif
 
 
-#define NODE_NO 0
+//#define NODE_NO 0
 //#define NODE_NO 1
+#define NODE_NO -1
 
 
 static void create_partitions (
@@ -64,7 +65,7 @@ void union_seed_packets(
  * \return An SP_MATRIX object containing the matrix of empty S_POINTS
  * and other information describing the matrix properties.
  */
-extern SP_MATRIX *create_spoint_matrix (
+SP_MATRIX *create_spoint_matrix (
   int *central_ws,   ///< A sorted list of central widths values for the matrix
   int n_ws,          ///< The number of central width values
   int *central_ns,   ///< A sorted list of central nsites values for the matrix
@@ -88,14 +89,13 @@ extern SP_MATRIX *create_spoint_matrix (
   int w_idx = 0;
   for (curr_w = min_w; curr_w <= max_w; curr_w++) {
     // get list of nsites0 for this width and add it to the matrix:
-    matrix[w_idx] =
-      create_spoint_row(curr_w, central_ws, n_ws, central_ns, n_ns, dataset);
+    matrix[w_idx] = create_spoint_row(curr_w, central_ws, n_ws, central_ns, n_ns, dataset);
     w_idx++;
   }
 
   // Create an SP_MATRIX object containing all the relevant information:
   SP_MATRIX *sp_matrix = NULL;
-  sp_matrix = (SP_MATRIX *)mymalloc(sizeof(SP_MATRIX));
+  sp_matrix = (SP_MATRIX *)mm_malloc(sizeof(SP_MATRIX));
   sp_matrix->matrix = matrix;
   sp_matrix->central_ws = central_ws;
   sp_matrix->n_ws = n_ws;
@@ -111,7 +111,7 @@ extern SP_MATRIX *create_spoint_matrix (
   return sp_matrix;
 } // create_spoint_matrix
 
-
+int info_cons0_alloc =  0;
 /**      
  * create_spoint_row
  *
@@ -120,7 +120,7 @@ extern SP_MATRIX *create_spoint_matrix (
  *
  * \return The newly created row, which can then be added to a matrix.
  */
-extern S_POINT *create_spoint_row (
+S_POINT *create_spoint_row (
   int width,         ///< The width of all s_points in the current row
   int *central_ws,   ///< A sorted list of central widths values for the matrix
   int n_ws,          ///< The number of central width values
@@ -147,8 +147,7 @@ extern S_POINT *create_spoint_row (
     s_point_row[col_idx].iseq = 0;
     s_point_row[col_idx].ioff = 0;
     s_point_row[col_idx].w0 = width;
-    s_point_row[col_idx].nsites0 = curr_nsites<max_nsites ?
-      curr_nsites : max_nsites;
+    s_point_row[col_idx].nsites0 = curr_nsites<max_nsites ? curr_nsites : max_nsites;
     s_point_row[col_idx].wgt_nsites = 0;
     s_point_row[col_idx].e_cons0 = NULL;
     s_point_row[col_idx].cons0 = NULL;
@@ -157,8 +156,7 @@ extern S_POINT *create_spoint_row (
     // Calculate the manhattan distance from the current (n,w) position to
     // the closest "central" (n,w) position...
 
-    // Find the distances of the central w and n closest to the current w and
-    // n:
+    // Find the distances of the central w and n closest to the current w and n:
     int w_idx;
     int curr_central_w;
     double lowest_w_dist = BIG;
@@ -175,7 +173,7 @@ extern S_POINT *create_spoint_row (
     double lowest_n_dist = BIG;
     for (n_idx = 0; n_idx < n_ns; n_idx++) {
       curr_central_n = central_ns[n_idx];
-      int curr_dist = (int)abs(curr_central_n - curr_nsites); 
+      int curr_dist = (int) fabs(curr_central_n - curr_nsites); 
       if (curr_dist < lowest_n_dist) {
         lowest_n_dist = curr_dist;
       }
@@ -186,7 +184,7 @@ extern S_POINT *create_spoint_row (
     // If branch_W search is to occur, then evaluation should occur at
     // every s_point. Otherwise it should only occur at central s_points:
     if (dataset->branch_params->w_branch && (lowest_n_dist == 0)) {
-      s_point_row[col_idx].evaluate = TRUE;
+      s_point_row[col_idx].evaluate = true;
     } else {
       // Initially only evaluate seeds at this s_point if the s_point is
       // central:
@@ -198,6 +196,7 @@ extern S_POINT *create_spoint_row (
     double factor = dataset->hs_decrease;
     int heap_size = MAX((int)max_hs/(pow(factor, manhattan_dist)), 1);
 
+    info_cons0_alloc++;
     Resize(s_point_row[col_idx].cons0, width+1, char);
     s_point_row[col_idx].cons0[0] = '\0';
     s_point_row[col_idx].seed_heap = create_heap(
@@ -206,7 +205,7 @@ extern S_POINT *create_spoint_row (
       (void *)copy_seed,
       (void (*)(void*))free_seed,
       (char* (*)(void*))get_str_seed,
-      (void *(*)(FILE *, void*))print_seed
+      (void (*)(FILE *, void*))print_seed
     );
   } // col_idx
 
@@ -218,7 +217,7 @@ extern S_POINT *create_spoint_row (
  * free_sp_matrix
  *
  */
-extern void free_sp_matrix (
+void free_sp_matrix (
   SP_MATRIX *sp_mat  ///< The object being destroyed.  
 ) {
   // Free the internal representation of the matrix...
@@ -258,18 +257,15 @@ extern void free_sp_matrix (
  * allocated to the spoint struct itself. Nor does is free the
  * e_cons0 or the cons0 referenced by the S_POINT.
  *
- * It assumes that "destroy_heap" will free the s_point's e_cons0 and cons0.
- * This is a valid assumption if the seed representation (for the s_point)
- * is from one of the seeds in the heap (eg. the "best" seed in the heap).
- * However if this assumption is violated, then the e_cons0 and cons0 for
- * this spoint WILL NOT BE FREED. This could potentially result in a
- * memory leak.
- *
  */
-extern void free_s_point (
+void free_s_point (
   S_POINT *sp        ///< The item being destroyed.
 ) {
   destroy_heap(sp->seed_heap);
+  if (sp->cons0) info_cons0_alloc--;
+  free(sp->e_cons0);
+  free(sp->cons0);
+  memset(sp, 0, sizeof(S_POINT));
 } // free_s_point
 
 
@@ -281,7 +277,7 @@ extern void free_s_point (
  * if it existed.
  *
  */
-extern void copy_s_point (
+void copy_s_point (
   S_POINT *s_point,  ///< The S_POINT being copied
   S_POINT *sp_copy   ///< The copy of the S_POINT - OUT
 ) {
@@ -296,8 +292,8 @@ extern void copy_s_point (
   sp_copy->wgt_nsites = s_point->wgt_nsites;
   sp_copy->evaluate = s_point->evaluate;
   sp_copy->sig = s_point->sig;
-  char *e_cons_copy = NULL;
-  Resize(e_cons_copy, s_point->w0, char);
+  uint8_t *e_cons_copy = NULL;
+  Resize(e_cons_copy, s_point->w0, uint8_t);
   int char_idx;
   for(char_idx = 0; char_idx < s_point->w0; char_idx++) {
     e_cons_copy[char_idx] = (s_point->e_cons0)[char_idx];
@@ -317,7 +313,7 @@ extern void copy_s_point (
  * Print a representation of a specified s_point. This is yet another
  * function that should be a "method" of the s_point class.
  */
-extern void print_s_point (
+void print_s_point (
   S_POINT *s_point,  ///< The object being printed.
   FILE *out          ///< The output destination.
 ) {
@@ -343,13 +339,12 @@ extern void print_s_point (
  * Transfer the scores of the best seeds in the S_POINT heaps into the
  * S_POINTs themselves.
  */
-extern void transfer_final_scores (
+void transfer_final_scores (
+  ALPH_T *alph, // alphabet
   SP_MATRIX *sp_matrix ///< This object
 ) {
-  // Proceed through the entire matrix, transfering the details for each
-  // S_POINT:
-  int row_idx;
-  int col_idx;
+  // Proceed through the entire matrix, transfering the details for each S_POINT:
+  int row_idx, col_idx;
   for (row_idx = 0; row_idx < sp_get_num_rows(sp_matrix); row_idx++) {
     S_POINT *curr_row = sp_matrix->matrix[row_idx];
     for (col_idx = 0; col_idx < sp_get_num_cols(sp_matrix); col_idx++) {
@@ -359,10 +354,12 @@ extern void transfer_final_scores (
       if (get_num_nodes(sp_heap) >= 1) {
         SEED *best_seed = (SEED *)(get_node(sp_heap, get_best_node(sp_heap)));
         curr_sp->score = get_seed_score(best_seed);
-        curr_sp->iseq = -1; // Seed does not correspond to a location in the dataset.
-        curr_sp->ioff = -1; // Seed does not correspond to a location in the dataset.
-        curr_sp->e_cons0 = get_e_seed(best_seed);
-        curr_sp->cons0 = get_str_seed(best_seed);
+        curr_sp->iseq = best_seed->iseq;
+        curr_sp->ioff = best_seed->ipos;
+        curr_sp->nsites0 = best_seed->nsites0;
+        curr_sp->e_cons0 = get_e_seed(alph, best_seed);
+        free(curr_sp->cons0);
+        curr_sp->cons0 = strdup(get_str_seed(best_seed));
       }
 
       /* If the seed heap of the current s_point is empty, then it could
@@ -379,14 +376,13 @@ extern void transfer_final_scores (
   } // row_idx
 } // transfer_final_scores
 
-
 /**
  * get_min_width
  *
  * \return The minimum width encompassed by this sp_matrix.
  *
  */
-extern int get_min_width (
+int get_min_width (
   SP_MATRIX *sp_mat  ///< The sp_matrix object.
 ) {
   return sp_mat->central_ws[0];
@@ -399,7 +395,7 @@ extern int get_min_width (
  * \return The maximum width of any s_point in the matrix.
  *
  */
-extern int get_max_width (
+int get_max_width (
   SP_MATRIX *sp_mat  ///< The sp_matrix object.
 ) {
   return sp_mat->central_ws[sp_mat->n_ws - 1];
@@ -412,7 +408,7 @@ extern int get_max_width (
  * \return An array of the central nsites-values in the matrix.
  *
  */
-extern int *get_central_ws (
+int *get_central_ws (
   SP_MATRIX *sp_mat  ///< The sp_matrix object.
 ) {
   return sp_mat->central_ws;
@@ -425,7 +421,7 @@ extern int *get_central_ws (
  * \return An array of the central nsites values in the matrix.
  *
  */
-extern int *get_central_ns (
+int *get_central_ns (
   SP_MATRIX *sp_mat  ///< The sp_matrix object.
 ) {
   return sp_mat->central_ns;
@@ -438,7 +434,7 @@ extern int *get_central_ns (
  * \return The minimum nsites encompassed by this sp_matrix.
  *
  */
-extern int get_min_nsites (
+int get_min_nsites (
   SP_MATRIX *sp_mat  ///< The sp_matrix object.
 ) {
   return sp_mat->central_ns[0];
@@ -451,7 +447,7 @@ extern int get_min_nsites (
  * \return The maximum nsites of any s_point in the matrix.
  *
  */
-extern int get_max_nsites (
+int get_max_nsites (
   SP_MATRIX *sp_mat  ///< The sp_matrix object.
 ) {
   return sp_mat->central_ns[sp_mat->n_ns - 1];
@@ -465,7 +461,7 @@ extern int get_max_nsites (
  * \return The number of "central" width values in the matrix:
  *
  */
-extern int get_num_central_widths (
+int get_num_central_widths (
   SP_MATRIX *sp_mat  ///< The sp_matrix object.
 ) {
   return sp_mat->n_ws;
@@ -479,7 +475,7 @@ extern int get_num_central_widths (
  * \return The number of "central" nsites values in the matrix:
  *
  */
-extern int get_num_central_nsites (
+int get_num_central_nsites (
   SP_MATRIX *sp_mat  ///< The sp_matrix object.
 ) {
   return sp_mat->n_ns;
@@ -494,7 +490,7 @@ extern int get_num_central_nsites (
  * is equal to the number of rows in the matrix.
  *
  */
-extern int sp_get_num_rows (
+int sp_get_num_rows (
   SP_MATRIX *sp_mat
 ) {
   return (get_max_width(sp_mat) - get_min_width(sp_mat) + 1);
@@ -508,7 +504,7 @@ extern int sp_get_num_rows (
  * This is equal to the number of columns in the matrix.
  *
  */
-extern int sp_get_num_cols (
+int sp_get_num_cols (
   SP_MATRIX *sp_mat
 ) {
   return (get_max_nsites(sp_mat) - get_min_nsites(sp_mat) + 1);
@@ -522,7 +518,7 @@ extern int sp_get_num_cols (
  * the matrix itself
  *
  */
-extern S_POINT **get_sp_matrix (
+S_POINT **get_sp_matrix (
   SP_MATRIX *sp_mat
 ) {
   return sp_mat->matrix;
@@ -538,7 +534,7 @@ extern S_POINT **get_sp_matrix (
  * \return An array of all S_POINTS with the specified seed width.
  *
  */
-extern S_POINT *get_sp_arr (
+S_POINT *get_sp_arr (
   SP_MATRIX *sp_mat, ///< The spoint matrix
   int seed_width     ///< The number of the desired row
 ) {
@@ -561,7 +557,7 @@ extern S_POINT *get_sp_arr (
  * \return A pointer to the apporpriate S_POINT object
  *
  */
-extern S_POINT *get_spoint (
+S_POINT *get_spoint (
   SP_MATRIX *sp_mat, ///< The spoint matrix
   int row_idx,       ///< The row of the desired spoint
   int col_idx        ///< The column of the desired spoint
@@ -578,7 +574,7 @@ extern S_POINT *get_spoint (
  * \return A pointer to the apporpriate S_POINT object
  *
  */
-extern S_POINT *get_s_point (
+S_POINT *get_s_point (
   SP_MATRIX *sp_mat, ///< The s_point matrix
   int width,         ///< The width the desired s_point
   int nsites         ///< The nsites of the desired s_point
@@ -598,7 +594,7 @@ extern S_POINT *get_s_point (
  * possibly different for each S_POINT, and was stored in each of the S_POINTs
  * prior to the calling of this function.
  */
-extern void update_s_point_heaps(
+void update_s_point_heaps(
   S_POINT *s_points, ///< The s_points whose heaps are to be updated.
   char *str_seed,    ///< The ascii representation of the seed being added to
                      ///< each of the S_POINT heaps.
@@ -606,7 +602,6 @@ extern void update_s_point_heaps(
 )
 {
 
-
   int sp_idx;
   for (sp_idx = 0; sp_idx < n_nsites0; sp_idx++) {
     /* The S_POINT object holds the score and the sequence for the current
@@ -621,17 +616,14 @@ extern void update_s_point_heaps(
 
     if (curr_sp.score > LITTLE) {
       // Construct a new seed from the score and the seed's string:
-      SEED *seed_for_heap = new_seed(str_seed, curr_sp.score);
+      SEED *seed_for_heap = new_seed(str_seed, curr_sp.score, curr_sp.iseq, curr_sp.ioff, curr_sp.nsites0);
 
       // Add the node to the current heap:
-      SEED *bumped_node = (SEED *)(add_node_heap(curr_sp.seed_heap,
-                                                 seed_for_heap));
+      SEED *bumped_node = (SEED *)(add_node_heap(curr_sp.seed_heap, seed_for_heap));
 
       // If the heap is full, then it will reject a node. If this happens,
       // then delete the "bumped" node, in order to avoid a memory leak:
-      if (bumped_node != NULL) {
-        free_seed(bumped_node);
-      }
+      if (bumped_node != NULL) free_seed(bumped_node);
     }
   } // s_point
 } // update_s_point_heaps
@@ -757,7 +749,7 @@ static void create_partitions (
  * no S_POINT had a score greater than LITTLE
  *
  */
-extern S_POINT *choose_em_starts (
+S_POINT *choose_em_starts (
   SP_MATRIX *sp_matrix, ///< The matrix of S_POINT objects to choose from.
   DATASET *data,     ///< Sequence dataset.
   MODEL *model,      ///< The nascent motif model. Contains parameters needed
@@ -793,6 +785,7 @@ extern S_POINT *choose_em_starts (
       // chosen s_points:
       int curr_sp_idx = n_spoints - 1;
       copy_s_point(best_sp, &(sp_arr[curr_sp_idx]));
+      info_cons0_alloc++;
     }
   } // part_idx
 
@@ -820,8 +813,10 @@ static S_POINT *search_matrix_partition (
 ) {
   // Consider each S_POINT in the partition. Find the S_POINT with the
   // smallest significance value...
-  double curr_best_sig = BIG;
-  S_POINT *curr_best_sp = NULL;
+  double best_sig = BIG;
+  double best_iseq = 0;
+  double best_ioff = 0;
+  S_POINT *best_sp = NULL;
   
   int curr_w, curr_n;
   for (curr_w = part->min_w; curr_w <= part->max_w; curr_w++) {
@@ -851,15 +846,22 @@ static S_POINT *search_matrix_partition (
 
         curr_sp->sig = curr_sig;
 
-        if (curr_sig <= curr_best_sig) {
-          curr_best_sig = curr_sig;
-          curr_best_sp = curr_sp;
+        if (
+          (curr_sig < best_sig) 
+	    || (curr_sig == best_sig && curr_sp->iseq < best_iseq) 
+	    || (curr_sig == best_sig && curr_sp->iseq == best_iseq && curr_sp->ioff < best_ioff) 
+          )
+        {
+          best_sig = curr_sig;
+          best_iseq = curr_sp->iseq;
+          best_ioff = curr_sp->ioff;
+          best_sp = curr_sp;
         }
       } // Only considering s_point if initialised
     } // curr_n
   } // curr_w
 
-  return curr_best_sp;
+  return best_sp;
 } // search_matrix_partition
 
 
@@ -869,7 +871,7 @@ static S_POINT *search_matrix_partition (
  * Print a representation of this matrix.
  *
  */
-extern void print_sp_matrix (
+void print_sp_matrix (
   SP_MATRIX *sp_mat, ///< The matrix being printed
   FILE *out          ///< The output destination.
 ) {
@@ -900,7 +902,7 @@ extern void print_sp_matrix (
  * reduce_across_heaps
  *
  * Do a reduction across an array of S_POINT heaps. For each S_POINT in the
- * array, all the seeds from the heaps on each node are combinded (using a
+ * array, all the seeds from the heaps on each node are combined (using a
  * union function). A heap containing the best seeds from every node is then
  * propogated to all nodes.
  *
@@ -919,40 +921,54 @@ void reduce_across_heaps(
   if (init==0){
     init = 1;
     SEED_PACKET seed_packet;
-    int block_lengths[4];
-    MPI_Aint displacements[4];
-    MPI_Aint address[4]; 
-    MPI_Datatype typelist[4];
+    int block_lengths[SEED_PACKET_LENGTH];
+    MPI_Aint displacements[SEED_PACKET_LENGTH];
+    MPI_Aint address[SEED_PACKET_LENGTH]; 
+    MPI_Datatype typelist[SEED_PACKET_LENGTH];
 
     // Build the derived datatype
     // set the types
     typelist[0]=MPI_DOUBLE;
     typelist[1]=MPI_INT;
     typelist[2]=MPI_INT;
-    typelist[3]=MPI_CHAR;
+    typelist[3]=MPI_INT;
+    typelist[4]=MPI_INT;
+    typelist[5]=MPI_INT;
+    typelist[6]=MPI_CHAR;
 
     // set number of elements of each type
-    block_lengths[0] = block_lengths[1] = block_lengths[2] = 1;
-    block_lengths[3] = MAXSITE;	// the maximum length of a seed
+    block_lengths[0] = 1;
+    block_lengths[1] = 1;
+    block_lengths[2] = 1;
+    block_lengths[3] = 1;
+    block_lengths[4] = 1;
+    block_lengths[5] = 1;
+    block_lengths[6] = MAXSITE+1;	// the maximum length of a seed
 
     // calculate the displacements
-    MPI_Address(&seed_packet.score, &address[0]);
-    MPI_Address(&seed_packet.width, &address[1]);
-    MPI_Address(&seed_packet.num_seed_packets, &address[2]);
-    MPI_Address(&seed_packet.seed, &address[3]);
+    MPI_Get_address(&seed_packet.score, &address[0]);
+    MPI_Get_address(&seed_packet.width, &address[1]);
+    MPI_Get_address(&seed_packet.iseq, &address[2]);
+    MPI_Get_address(&seed_packet.ipos, &address[3]);
+    MPI_Get_address(&seed_packet.num_seed_packets, &address[4]);
+    MPI_Get_address(&seed_packet.nsites0, &address[5]);
+    MPI_Get_address(&seed_packet.seed, &address[6]);
     displacements[0]=0;
     displacements[1]=address[1]-address[0];
     displacements[2]=address[2]-address[0];
     displacements[3]=address[3]-address[0];
+    displacements[4]=address[4]-address[0];
+    displacements[5]=address[5]-address[0];
+    displacements[6]=address[6]-address[0];
 
     // create the derived type
-    MPI_Type_struct(4, block_lengths, displacements, typelist, &seed_packet_type);
+    MPI_Type_create_struct(SEED_PACKET_LENGTH, block_lengths, displacements, typelist, &seed_packet_type);
 
     // commit the derived type
     MPI_Type_commit(&seed_packet_type);
 
     // set the MPI reduction operation
-    MPI_Op_create(union_seed_packets, FALSE, &union_seed_packets_op);
+    MPI_Op_create(union_seed_packets, false, &union_seed_packets_op);
   } // initialise MPI
 
   // do a reduction for each s_point in the s_point list
@@ -980,38 +996,40 @@ void reduce_across_heaps(
       packets[i_packet].score = get_seed_score(curr_seed);
       // set the width of the string
       packets[i_packet].width = get_width(curr_seed);
+      // set the iseq of the string
+      packets[i_packet].iseq= curr_seed->iseq;
+      // set the ipos of the string
+      packets[i_packet].ipos = curr_seed->ipos;
+      // set the nsites0 of the score
+      packets[i_packet].nsites0 = curr_seed->nsites0;
       // set the seed
       char *seed_str = get_str_seed(curr_seed);
       strcpy(packets[i_packet].seed, seed_str); 
     }
 
-/*
     // print the packets before the reduction
-    if (mpMyID() == NODE_NO){
+    if (NODE_NO >= 0 && mpMyID() == NODE_NO){
       fprintf(stdout, "BEFORE\n");
       for (i_packet = 0; i_packet < max_heap_size; i_packet++)
-      fprintf(stdout, "node %d packet %d score= %g width= %i seed= %s\n",
+        fprintf(stdout, "BEFORE node %d packet %d score= %g width= %i seed= %s nsites0= %d\n",
                        mpMyID(), i_packet, packets[i_packet].score,
-                       packets[i_packet].width, packets[i_packet].seed);
+                       packets[i_packet].width, packets[i_packet].seed, packets[i_packet].nsites0);
       fflush(stdout);
     }
-*/
 
     // Do the reduction
     MPI_Allreduce((void *)&packets, (void *)&best_packets, max_heap_size,
                     seed_packet_type, union_seed_packets_op, MPI_COMM_WORLD);
 
-/*
     // print the packets after the reduction
-    if (mpMyID() == NODE_NO){
+    if (NODE_NO >= 0 && mpMyID() == NODE_NO){
       fprintf(stdout, "AFTER\n");
       for (i_packet = 0; i_packet < max_heap_size; i_packet++)
-      fprintf(stdout, "node %d packet %d score= %g width= %i seed= %s\n",
+        fprintf(stdout, "AFTER node %d packet %d score= %g width= %i seed= %s iseq %d ioff %d nsites0= %d\n",
                        mpMyID(), i_packet, best_packets[i_packet].score,
-                       best_packets[i_packet].width, best_packets[i_packet].seed);
+                       best_packets[i_packet].width, best_packets[i_packet].seed, best_packets[i_packet].iseq, best_packets[i_packet].ipos, best_packets[i_packet].nsites0);
       fflush(stdout);
     }
-*/
 
     // Unpack the best seed packets into the heap
 
@@ -1020,17 +1038,25 @@ void reduce_across_heaps(
 
     // Add the best seeds to the heap
     for (i_packet = 0; i_packet < num_seed_packets; i_packet++){
-      double score =  best_packets[i_packet].score;
+      double score = best_packets[i_packet].score;
       char *seed_str = best_packets[i_packet].seed;
-      SEED *best_seed = new_seed(seed_str, score);
+      int iseq = best_packets[i_packet].iseq;
+      int ipos = best_packets[i_packet].ipos;
+      int nsites0 = best_packets[i_packet].nsites0;
+      SEED *best_seed = new_seed(seed_str, score, iseq, ipos, nsites0);
       //SEED *bumped_seed = (SEED *)(add_node_heap(seed_heap, best_seed));
-      (void *)(add_node_heap(seed_heap, best_seed));
+      (void)(add_node_heap(seed_heap, best_seed));
+    }
+
+    // Set nsites0 in the starting point in case it changed.
+    if (get_num_nodes(seed_heap) > 0) {
+      SEED *best_seed = (SEED *)get_node(seed_heap, get_best_node(seed_heap));
+      s_points[sp_idx].nsites0 = best_seed->nsites0;
     }
   } // end n_nsites0
 
 } // reduce_across_heaps
 
-
 /**
  * union_seed_packets
  *
@@ -1054,7 +1080,7 @@ void union_seed_packets(void *f_data, void *f_result, int *f_length,
       (void *)copy_seed,
       (void (*)(void*))free_seed,
       (char* (*)(void*))get_str_seed,
-      (void *(*)(FILE *, void*))print_seed
+      (void (*)(FILE *, void*))print_seed
     );
   
   // get the number of seed_packets in f_data
@@ -1065,7 +1091,10 @@ void union_seed_packets(void *f_data, void *f_result, int *f_length,
     // get the data seed
     char *data_seed_str = ((SEED_PACKET *)f_data + i)->seed;
     double data_score = ((SEED_PACKET *)f_data + i)->score;
-    SEED *data_seed = new_seed(data_seed_str, data_score);
+    int data_iseq = ((SEED_PACKET *)f_data + i)->iseq;
+    int data_ipos = ((SEED_PACKET *)f_data + i)->ipos;
+    int data_nsites0 = ((SEED_PACKET *)f_data + i)->nsites0;
+    SEED *data_seed = new_seed(data_seed_str, data_score, data_iseq, data_ipos, data_nsites0);
     // add the seeds to the heap
     bumped_seed = (SEED *)(add_node_heap(heap, data_seed)); 
   } 
@@ -1076,7 +1105,10 @@ void union_seed_packets(void *f_data, void *f_result, int *f_length,
     // get the result seed
     char *result_seed_str = ((SEED_PACKET *)f_result + i)->seed;
     double result_score = ((SEED_PACKET *)f_result + i)->score;
-    SEED *result_seed = new_seed(result_seed_str, result_score);
+    int result_iseq = ((SEED_PACKET *)f_result + i)->iseq;
+    int result_ipos = ((SEED_PACKET *)f_result + i)->ipos;
+    int result_nsites0 = ((SEED_PACKET *)f_result + i)->nsites0;
+    SEED *result_seed = new_seed(result_seed_str, result_score, result_iseq, result_ipos, result_nsites0);
     // add the seeds to the heap
     bumped_seed = (SEED *)(add_node_heap(heap, result_seed)); 
   }
@@ -1093,8 +1125,10 @@ void union_seed_packets(void *f_data, void *f_result, int *f_length,
     int heap_idx = i + 1;
     // get the node
     SEED *curr_seed = get_node(heap, heap_idx);
-    //double score = get_seed_score(curr_seed);
     ((SEED_PACKET *)f_result + i)->score = get_seed_score(curr_seed);
+    ((SEED_PACKET *)f_result + i)->iseq = curr_seed->iseq;
+    ((SEED_PACKET *)f_result + i)->ipos = curr_seed->ipos;
+    ((SEED_PACKET *)f_result + i)->nsites0 = curr_seed->nsites0;
     char *seed_str = get_str_seed(curr_seed);
     strcpy(((SEED_PACKET *)f_result + i)->seed, seed_str);
   }
@@ -1111,7 +1145,7 @@ void union_seed_packets(void *f_data, void *f_result, int *f_length,
  * \return A heap containing the seeds from EVERY s_point heap in the matrix
  */
 
-extern HEAP *create_heap_from_sp_matrix (
+HEAP *create_heap_from_sp_matrix (
   SP_MATRIX *sp_mat    // the matrix of s_points 
 ) {
 
@@ -1137,7 +1171,7 @@ extern HEAP *create_heap_from_sp_matrix (
       (void *)copy_seed,
       (void (*)(void*))free_seed,
       (char* (*)(void*))get_str_seed,
-      (void *(*)(FILE *, void*))print_seed
+      (void (*)(FILE *, void*))print_seed
     );
 
   // add the seeds to the heap
