@@ -36,6 +36,7 @@
 #include "macros.h"
 #include "matrix.h"
 #include "motif.h"	// for motif2consensus
+#include "motif-db.h"
 #include "xml-out.h"
 #include "projrel.h"	// for ARCHIVE_DATE
 
@@ -72,8 +73,8 @@ void mmcheckspaceleak(void);
 // Prior for creating higher-order background model.
 #define MEME_BG_PRIOR 1
 
-// Maximum allowed kmer size.
-#define MAX_KMER 6
+// Maximum allowed order.
+#define MAX_ORDER 5
 
 // Size for switching from list to bit table.
 #define LISTSIZE 200
@@ -86,6 +87,9 @@ void mmcheckspaceleak(void);
 
 #define FLOAT_EPS 1e-10
 
+// Number of significant digits after rounding.
+#define STREME_RND 10
+
 // Print the current memory usage.
 // Activate with: make CFLAGS=" -O3 -DMEMORY_MONITOR" <program>
 #ifdef MEMORY_MONITOR
@@ -99,18 +103,24 @@ void mmcheckspaceleak(void);
   #define PRINT_MEMORY_USAGE()
 #endif
 
-// Objective function types.
-typedef enum {DE, CD} OBJFUN_T;
+// Objective function type.
+typedef enum {DE, CD, NO_OBJFUN} OBJFUN_T;
 
 // Saving matches type.
 typedef enum {NONE, ZOOPS, PASSING} SAVE_MATCHES_T;
 
+// Site positional distribution alignment type.
+typedef enum {LEFT, CENTER, RIGHT} SEQ_ALIGN_T;
+
+// Threshold type.
+typedef enum {EVALUE, QVALUE, PVALUE} THRESH_T;
+
 DEXTERN(OBJFUN_T, DEFAULT_OBJFUN, DE);
 DEXTERN(int, DEFAULT_ALPHABET_TYPE, Dna);
-DEXTERN(int, DEFAULT_DNA_KMER, 3);
-DEXTERN(int, DEFAULT_RNA_KMER, 3);
-DEXTERN(int, DEFAULT_PROT_KMER, 1);
-DEXTERN(int, DEFAULT_CUSTOM_KMER, 1);
+DEXTERN(int, DEFAULT_DNA_ORDER, 2);
+DEXTERN(int, DEFAULT_RNA_ORDER, 2);
+DEXTERN(int, DEFAULT_PROT_ORDER, 0);
+DEXTERN(int, DEFAULT_CUSTOM_ORDER, 0);
 DEXTERN(int, DEFAULT_MINWIDTH, 8);
 DEXTERN(int, DEFAULT_MAXWIDTH, 15);
 DEXTERN(int, DEFAULT_NEVAL, 25);
@@ -118,6 +128,7 @@ DEXTERN(int, DEFAULT_NREF, 4);
 DEXTERN(int, DEFAULT_NITER, 20);
 DEXTERN(int, DEFAULT_NMOTIFS, 5);		// use pvt by default; kicks in if pvt == 0
 DEXTERN(int, DEFAULT_TIME, 0);			// must be >= 0; 0->no limit
+DEXTERN(double, DEFAULT_EVT, 0.05);
 DEXTERN(double, DEFAULT_PVT, 0.05);
 DEXTERN(int, DEFAULT_PATIENCE, 3);		// quit after this many consecutive non-significant motifs 
 DEXTERN(double, DEFAULT_MIN_PAL_RATIO, 0.85);
@@ -130,6 +141,7 @@ DEXTERN(int, DEFAULT_IGNORE_DEPTH, 5);		// Don't enforce score cutoff below this
 DEXTERN(int, DEFAULT_SEED, 0);
 DEXTERN(double, DEFAULT_PSEUDOCOUNT, 0);
 DEXTERN(double, DEFAULT_LOG_PVALUE, 0);
+DEXTERN(int, DEFAULT_CCUT, 0);
 DEXTERN(int, DEFAULT_VERBOSITY, NORMAL_VERBOSE);
 DEXTERN(VERBOSE_T, verbosity, NORMAL_VERBOSE);	// for MEME routines
 
@@ -214,7 +226,7 @@ EXTERN int COMPINDEX[MAX_ALENGTH+1];
   { \
     int i, j; \
     (model)->min_possible_score = 0; \
-    for (j=0; j<w; j++) { \
+    for (j=0; j<(model)->width; j++) { \
       double min_possible_score_j = 0; \
       for (i=0; i<(model)->alen; i++) { \
 	(model)->pssm[i][j] = ( (order) == 0 ? log((model)->probs[i][j] / (background)[i]) : log((model)->probs[i][j]) ) / log(2); \
@@ -243,22 +255,35 @@ EXTERN int COMPINDEX[MAX_ALENGTH+1];
     } \
   }
 
-// Print the MOTIF in MEME format. Prints the
-// hold-out p-value in the "P=" field (the "E=" field in MEME output).
+// Print the MOTIF in MEME format.
+//  1) If have_holdout, prints the hold-out p-value in the "P=" field (the "E=" field in MEME output),
+//  2) else, prints the training set p-value in the "S=" field (the "E=" field in MEME output).
 // The "nsites=" field is set to the SUM of the positive test and hold-out matches.
 // Uses the single-letter consensus for the motif name, prepending "text".
-#define PRINT_MOTIF(model, motifno, index, text, stream) \
+#define PRINT_MOTIF(model, motifno, index, text, stream, have_holdout) \
   { \
     int _i, _j; \
     double m1, e1, prec=1; \
-    exp10_logx((model)->test_log_pvalue/log(10), m1, e1, prec); \
+    char *pv_str; \
+    if ((have_holdout)) { \
+      if ((index) == -1) { \
+        exp10_logx((model)->test_log_evalue/log(10), m1, e1, prec); \
+        pv_str = "E="; \
+      } else { /* cand */ \
+	exp10_logx((model)->test_log_pvalue/log(10), m1, e1, prec); \
+        pv_str = "P="; \
+      } \
+    } else { \
+      exp10_logx((model)->train_log_pvalue/log(10), m1, e1, prec); \
+      pv_str = "S="; \
+    } \
     if (*(text) != '\0') { \
       fprintf((stream), "MOTIF %s%d-%g-%s\n", (text), (motifno), (double) (index), (model)->consensus); \
     } else { \
       fprintf((stream), "MOTIF %d-%s STREME-%d\n", (motifno), (model)->consensus, (motifno)); \
     } \
-    fprintf((stream), "letter-probability matrix: alength= %d w= %d nsites= %d P= %3.1fe%+04.0f\n", \
-      (model)->alen, (model)->width, (model)->train_pos_count + (model)->test_pos_count, m1, e1); \
+    fprintf((stream), "letter-probability matrix: alength= %d w= %d nsites= %d %s %3.1fe%+04.0f\n", \
+      (model)->alen, (model)->width, (model)->train_pos_count + (model)->test_pos_count, pv_str, m1, e1); \
     for (_j=0; _j<(model)->width; _j++) { \
       for (_i=0; _i<(model)->alen; _i++) { \
         fprintf((stream), " %.6f", (model)->probs[_i][_j]); \
@@ -344,7 +369,8 @@ EXTERN int COMPINDEX[MAX_ALENGTH+1];
       fprintf(stderr, "ERROR: unknown objfun in %s\n", text); \
       exit(EXIT_FAILURE); \
     } \
-    RND((log_pvalue), RNDDIG, (log_pvalue)); \
+    RND2((log_pvalue), STREME_RND, (log_pvalue)); \
+    (log_pvalue) = MIN(0, (log_pvalue)); \
   }
 
 //
@@ -396,6 +422,15 @@ typedef struct {
   int dtc;		// average distance between site centers and sequence centers
 } Site;
 
+// Sequence with a passing site.
+typedef struct {
+  int dbno;		// sequence database: 0,1
+  int seqno;		// sequence number within the database
+  Uchar *desc;		// Sequence ID
+  double score;		// Best score of sequence
+  BOOL is_tp;		// True if primary sequence, False if control sequence.
+} Passing_seq;
+
 // Motif MODEL.
 // MAX_CUST_ALEN = 10 digits + 52 letters + 4 other characters (Resize(name, i+RCHUNK, char) = 66
 #define MAX_CUST_ALEN 66
@@ -414,17 +449,29 @@ typedef struct {
   Uint train_neg_count;				// best number of negatives
   double train_dtc;				// mean distance from site center to sequence center
   double score_threshold;			// best threshold for separating pos and neg sites
-  double train_log_pvalue;			// best p-value on training set
+  double train_log_pvalue;			// best log p-value on training set
   double train_ratio;				// ratio of positive to negative counts on training set
+  int n_eff_tests;				// number of successful multiple tests (if no hold-out set)
+  MOTIF_T *motif;				// the MEME-formatted motif entry
+  int db_idx;					// index of DB containing motif
   // hold-out set
   Uint test_pos_count;				// best number of hold-out positives
   Uint test_neg_count;				// best number of hold-out negatives 
   double test_dtc;				// best hold-out mean distance from site center to sequence center
-  double test_log_pvalue;			// best p-value on hold out set
+  double test_log_pvalue;			// best log p-value on hold-out set
+  double test_log_evalue; 			// best log E-value on hold-out set
+  double test_log_qvalue;	 		// log q-value on hold-out set (used by SEA only)
   double test_ratio;				// ratio of positive to negative counts on hold-out set
   // sites
   Site *matches;				// positive and negative matches to erase (malloced per model)
   Uint nmatches;				// length of matches list
+  int total_sites;				// total number positive sequences with PASSING sites
+  int *site_distr;				// counts of PASSING sites; sequences are aligned to center of longest sequence
+  int max_sites;				// maximum number of PASSING sites in any positive sequence
+  int *site_hist;				// number of PASSING positive sequences with each number of PASSING sites <= max_sites
+  // matching sequences
+  Passing_seq *passing_seqs;			// sequences with a passing site
+  int npassing;					// number of sequences with a passing site
   // other
   BOOL is_palindromic;				// model is palindromic
   double ed;					// palindrome edit distance
@@ -501,7 +548,8 @@ typedef struct options {
   char *negfile;			// negative sequences filename
   int totallength;			// truncate each sequence set (+/-) to this length
   OBJFUN_T objfun;			// objective function
-  int kmer;				// preserve k-mers when shuffle sequences
+  int order;				// use this order background model and sequence shuffling
+  char *bfile;                          // name of MEME-style background model file
   ALPHABET_T alphabet_type;		// alphabet type (Dna, Rna, Protein, Custom)
   ALPH_T *alph;				// alphabet for dataset 
   char *alph_file;			// custom alphabet file
@@ -511,7 +559,9 @@ typedef struct options {
   int neval;				// number of initial seeds of each width to evaluate
   int nref;				// number of evaluated seeds to refine
   int niter;				// maximum number of refinement iterations/seed
-  double pvt;				// p-value threshold for motifs
+  //double pvt;				// p-value threshold for motifs
+  double thresh;			// significance threshold for reporting enriched motifs
+  THRESH_T thresh_type;			// statistic for significance threshold (E-, q-, or p-value)
   int patience;				// quit after this many consecutive non-significant motifs
   int nmotifs;				// number of motifs to find
   double time;				// try to quit before <time> CPU seconds consumed
@@ -527,19 +577,25 @@ typedef struct options {
   char *description;			// value of --desc or the contents of file named dfile
   char *dfile;				// name of file containing description
   BOOL experimental;			// set true if compiled with EXP defined
+  char *align_txt;			// align sequences left/center/right for distr plots
+  SEQ_ALIGN_T align;			// align sequences left/center/right for distr plots
 //
 // Derived options:
+  int kmer;				// use this for sequence shuffling
   FILE *text_output;
   char *commandline;
   double **pv_cache;			// cache for p-values, indexed by pos_count + neg_count
   int cache_length;			// length of pv_cache array
 //
-// Options for ENR only:
+// Options for SEA only:
+  FILE *sequences_tsv_output;		// name of TSV file for the passing sequences of each significant motif
+  BOOL noseqs;				// don't output passing sequences if true
   ARRAYLST_T *motif_sources;            // filenames of the motif library
   ARRAYLST_T *include_patterns;         // Wildcard patterns of motif names to include
   ARRAYLST_T *exclude_patterns;         // Wildcard patterns of motif names to exclude
   double pseudocount;                   // add to the motif frequency counts
-  char* bg_source;                      // file to get base freqs from
+// Options for fasta-holdout-set only:
+  int ccut;				// centrally trim sequences longer than this
 } STREME_OPTIONS_T;
 
 void shuffle_multiseq(
@@ -552,24 +608,37 @@ Multiseq *read_pos_neg_seqs(
   STREME_OPTIONS_T *options,
   BOOL do_rc,                           // append reverse complement of pos+neg sequences
                                         // negative sequences by shuffling positive sequences
-  Multiseq **test_multiseq_ptr          // hold-out set
+  BOOL allow_ambigs,                    // don't convert ambiguous characters to SEPARATOR
+  BOOL no_trim,                         // don't trim sequences to average length
+  BOOL set_back,                        // set the background in the multiseq objects
+  ARRAY_T *background,                  // IN MEME-style background model
+  Multiseq **test_multiseq_ptr          // OUT hold-out set
 );
 
 void set_multiseq_background(
   STREME_OPTIONS_T *options,
   Multiseq *multiseq,
   ALPH_T *alph,
-  BOOL do_rc
+  BOOL do_rc,
+  ARRAY_T *back                 // MEME-style background model (or NULL)
+);
+
+void set_logcumback(
+  STREME_OPTIONS_T *options,
+  Multiseq *multiseq
 );
 
 void append_to_multiseq(
   Multiseq *multiseq,           // old sequences
   Multiseq *multiseq2,          // sequences to append
+  BOOL no_trim,
   BOOL is_ho
 );
 
 void append_rc_to_multiseq (Multiseq *multiseq);
 
+void print_multiseq(Multiseq *multiseq); 
+
 void score_model_pssm(
   STREME_OPTIONS_T *options,            // STREME options
   Multiseq *multiseq,                   // the positive and negative sequences
@@ -578,7 +647,22 @@ void score_model_pssm(
   BOOL set_pvalue,                      // compute the enrichment p-value of the model
   SAVE_MATCHES_T save_matches,          // ZOOPS: save best match (sorted) in every sequence (positive or negative) for nesting
                                         // PASSING: save all matches with score >= model->score_threshold for erasing
-  BOOL is_ho                            // data is the hold-out set
+  BOOL is_ho,                           // data is the hold-out set
+  BOOL use_cache                        // use the p-value cache
+);
+
+void get_passing_sequences(
+  STREME_OPTIONS_T *options,    // STREME options
+  Model *model,                 // the model, including its PASSING sites (see score_model_pssm)
+  Multiseq *multiseq,           // the sequences
+  BOOL append,                  // append passing sites to model if true
+  BOOL sort                     // sort the passing sequences by score
+);
+
+void get_site_distr(
+  Model *model,                 // The model, including its PASSING sites (see score_model_pssm)
+  Multiseq *multiseq,           // The sequences
+  SEQ_ALIGN_T align             // Align sequences left, center or right.
 );
 
 int compare_site_score(
@@ -586,6 +670,11 @@ int compare_site_score(
   const void *v2
 );
 
+int compare_sequence_score(
+  const void *v1,
+  const void *v2
+);
+
 Uint initialize_st_alphabet(
   ALPH_T *alph                  // the MEME-style alphabet
 );
@@ -594,6 +683,11 @@ char *get_single_letter_consensus(
   Model *model
 );
 
+//
+// Read the contents of a description file.
+//
+char *get_description_file(char *filename);
+
 // Imported from st_mapfile.c
 void mmcheckspaceleak(void);
 
