@@ -1,12 +1,14 @@
 /***********************************************************************
 *                                                                      *
-*       MetaMEME						       *
-*	Date:		23/4/2002				       *
-*       Author: 	Timothy L. Bailey                              *
-*	Copyright:	University of Queensland		       *
+*       MetaMEME                                                       *
+*       Date:           23/4/2002                                      *
+*       Author:         Timothy L. Bailey                              *
+*       Copyright:      University of Queensland                       *
 *                                                                      *
 ***********************************************************************/
 
+#include <assert.h>
+
 #include "matrix.h"
 #include "karlin.h"
 #include "rdb-matrix.h"
@@ -23,7 +25,6 @@
 /* dayhoff PAM 1 (matrix; order of alphabet: ACDEFGHIKLMNPQRSTVWY */
 /* dayhoff_ij = Pr(amino acid j --> amino acid i | time=1)
               = Pr(i | j, t=1) */
-char *pam_prot_alphabet = "ACDEFGHIKLMNPQRSTVWY";
 double dayhoff[20][20] = {
   { 9867, 3, 10, 17, 2, 21, 2, 6, 2, 4, 6, 9, 22, 8, 2, 35, 32, 18, 0, 2},
   { 1, 9973, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 1, 5, 1, 2, 0, 3},
@@ -48,7 +49,6 @@ double dayhoff[20][20] = {
 };
 
 /* transition/transversion PAM 1 matrix; alphabet order "ACGT" */
-char *pam_dna_alphabet = "ACGT";
 double trans[4][4] = {
   { 9900, 20, 60, 20}, 
   { 20, 9900, 20, 60},
@@ -90,39 +90,41 @@ double pam_prot_freq[] = {
 
 /**********************************************************************/
 /*
-	reorder_matrix
+        reorder_matrix
 
-	Reorder a matrix from the alphabet order in alpha1
-	to the alphabet order of alpha2.  The letters in alph1 must be
-	a superset of those in alpha2.  Rows and columns corresponding
-	to letters missing from alpha2 are discarded. 
+        Reorder a matrix from the alphabet order in alpha1
+        to the alphabet order of alpha2.  The letters in alph1 must be
+        a superset of those in alpha2.  Rows and columns corresponding
+        to letters missing from alpha2 are discarded. 
 
-	Returns a reordered square matrix of size length(alpha2).
+        Returns a reordered square matrix of size length(alpha2).
 */
 /**********************************************************************/
 MATRIX_T *reorder_matrix(
-  char *alpha1,				/* current alphabet */
-  char *alpha2,				/* new alphabet; must be subset */
-  MATRIX_T *in_matrix			/* matrix to reorder */
+  const char *source_alphabet_string,                           /* current alphabet */
+  const ALPH_T *target_alphabet,                           /* new alphabet; must be subset */
+  MATRIX_T *in_matrix                   /* matrix to reorder */
 )
 {
   int i, j;
-  int alen1 = strlen(alpha1);
-  int alen2 = strlen(alpha2);
+  int alen1 = strlen(source_alphabet_string);
+  int alen2 = alph_size_core(target_alphabet);
   MATRIX_T *out_matrix;
 
   if (alen2 > alen1) 
-    die("The new alphabet %s must be a subset of the old alphabet %s.\n", alpha2, alpha1);
+    die("The new alphabet %s must be a subset of the old alphabet %s.\n", alph_name(target_alphabet), source_alphabet_string);
 
   out_matrix = allocate_matrix(alen2, alen2);
-  for (i=0; i<alen2; i++) {
-    int ii = strchr(alpha1, alpha2[i]) - alpha1;
-    for (j=0; j<alen2; j++) {
+
+  //TODO: should probably be rewritten to use alph_index
+  for (i = 0; i < alen2; i++) {
+    int ii = strchr(source_alphabet_string, alph_char(target_alphabet, i)) - source_alphabet_string;
+    for (j = 0; j < alen2; j++) {
       int jj;
-      char *ptr = strchr(alpha1, alpha2[j]);
+      char *ptr = strchr(source_alphabet_string, alph_char(target_alphabet, j));
       if (!ptr)
-        die("The new alphabet %s must be a subset of the old alphabet %s\n", alpha2, alpha1);
-      jj = ptr - alpha1;
+        die("The new alphabet %s must be a subset of the old alphabet %s\n", alph_name(target_alphabet), source_alphabet_string);
+      jj = ptr - source_alphabet_string;
       set_matrix_cell(i, j, get_matrix_cell(ii, jj, in_matrix), out_matrix);
     }
   }
@@ -131,45 +133,47 @@ MATRIX_T *reorder_matrix(
 
 /**********************************************************************/
 /*
-       	gen_pam_matrix
+        gen_pam_matrix
 
-	Generate a PAM joint probability or log-odds score matrix.  
+        Generate a PAM joint probability or log-odds score matrix.  
 
-	If logodds is false, generates the joint probability matrix:
-	  M_ij = Pr(i,j | time = dist).
+        If logodds is false, generates the joint probability matrix:
+          M_ij = Pr(i,j | time = dist).
         If logodds is true, generates a log-odds scoring matrix:
-	  M_ij = factor * log(Pr(i,j | time = dist)/ Pr(i)Pr(j)),
-	where 
+          M_ij = factor * log(Pr(i,j | time = dist)/ Pr(i)Pr(j)),
+        where 
           factor = 2/log(2), 2 <= dist < 170, 
-	           3/log(2), 170 <= dist.
+                   3/log(2), 170 <= dist.
 
-	This is adapted from MEME code.  I'm not sure where I got
-	the original conditional probabilities--I think from a GCG file.
-	The log-odds matrices generated are close, but not exactly the
-	same as those generated by "pam" Version 1.0.6 that BLAST and
-	everyone else seems to have used to create PAM files, 
-	and I have not been able to locate a copy of that program. 
-	I have no idea where I got the "transversion/transition matrix"
-	that is used for making DNA "pam" matrices.
+        This is adapted from MEME code.  I'm not sure where I got
+        the original conditional probabilities--I think from a GCG file.
+        The log-odds matrices generated are close, but not exactly the
+        same as those generated by "pam" Version 1.0.6 that BLAST and
+        everyone else seems to have used to create PAM files, 
+        and I have not been able to locate a copy of that program. 
+        I have no idea where I got the "transversion/transition matrix"
+        that is used for making DNA "pam" matrices.
 
-	Returns the matrix.
+        Returns the matrix.
 */
 /**********************************************************************/
 
-extern MATRIX_T* gen_pam_matrix(
-  BOOLEAN_T dna,		/* DNA alphabet if true; default: protein */
-  int dist,			/* PAM distance */
-  BOOLEAN_T logodds		/* true: generate log-odds matrix 
-				   false: generate target frequency matrix 
-				*/
+MATRIX_T* gen_pam_matrix(
+  ALPH_T* alph,                  /* alphabet */
+  int dist,                     /* PAM distance */
+  bool logodds             /* true: generate log-odds matrix 
+                                   false: generate target frequency matrix 
+                                */
 )
 {
+  //TODO: this only works with DNA and PROTEIN alphabets
+  assert(alph != NULL && (alph_is_builtin_dna(alph) || alph_is_builtin_protein(alph)));
   int i, j;
   MATRIX_T *matrix, *mul;
-  double *pfreq = dna ? pam_dna_freq : pam_prot_freq;	/* standard frequencies */
-  char *alpha = dna ? pam_dna_alphabet : pam_prot_alphabet;	/* standard alphabet */
-  int alen = strlen(alpha);				/* length of standard alphabet */
-  double factor = dist < 170 ? 2/log(2) : 3/log(2);	/* same as in "pam" Version 1.0.6 */
+  bool dna = alph_is_builtin_dna(alph);
+  double *pfreq = dna ? pam_dna_freq : pam_prot_freq;   // standard frequencies
+  int alen = alph_size_core(alph);  // length of standard alphabet
+  double factor = dist < 170 ? 2/log(2) : 3/log(2);     // same as in "pam" Version 1.0.6
 
   /* create the array for the joint probability matrix */
   matrix = allocate_matrix(alen, alen);
@@ -215,80 +219,80 @@ extern MATRIX_T* gen_pam_matrix(
 
 /**********************************************************************/
 /*
-	read_score_matrix
+        read_score_matrix
 
-	Read in a score matrix in the format used by BLAST. 
+        Read in a score matrix in the format used by BLAST. 
 
-	Only reads in the standard letters in the alphabet.
+        Only reads in the standard letters in the alphabet.
 */
 /**********************************************************************/
 MATRIX_T *read_score_matrix(
-  char *score_filename,			/* name of score file */
-  char **alpha1				/* alphabet in score matrix */
+  char *score_filename,                 /* name of score file */
+  char **alpha1                         /* alphabet in score matrix */
 )
 {
   int i;
-  char *alpha;				/* alphabet in file */
+  char *alpha;                          /* alphabet in file */
   int alen;
   FILE *score_file;
   RDB_MATRIX_T *rdb_matrix;
 
   /* open the score file */
-  if (open_file(score_filename, "r", FALSE, "score", "substitution scores", &score_file) == 0) 
+  if (open_file(score_filename, "r", false, "score", "substitution scores", &score_file) == 0) 
     exit(1);
 
   /* read in the score file */
-  rdb_matrix = read_rdb_matrix(" ", FALSE, 0, FALSE, NULL, score_file);
+  rdb_matrix = read_rdb_matrix(" ", false, 0, false, NULL, score_file);
 
   /* get alphabet */
   alen = get_num_strings(rdb_matrix->col_names);
   alpha = (char *)mm_malloc(sizeof(char) * (alen+1));
   for (i=0; i<alen; i++) alpha[i] = get_nth_string(i, rdb_matrix->col_names)[0];
   alpha[i] = '\0';
-  *alpha1 = alpha;			/* return alphabet */
+  *alpha1 = alpha;                      /* return alphabet */
 
   return(rdb_matrix->matrix);
 } /* read_score_matrix */
 
 /**********************************************************************/
 /*
-	get_score_matrix
+        get_score_matrix
 
-	Get a letter substitution scoring matrix.
+        Get a letter substitution scoring matrix.
 
-	If a filename is given, the scoring matrix is read from the file.
-	Otherwise, a PAM matrix of the given type and  evolutionary 
-	distance is generated.
+        If a filename is given, the scoring matrix is read from the file.
+        Otherwise, a PAM matrix of the given type and  evolutionary 
+        distance is generated.
 
-	The matrix is reordered to conform to the standard alphabet.  This
-	alphabet must be a subset of the alphabet in the file (if the
-	matrix was read from a file).  Any rows and columns for letters
-	not in the standard alphabet are discarded.
+        The matrix is reordered to conform to the standard alphabet.  This
+        alphabet must be a subset of the alphabet in the file (if the
+        matrix was read from a file).  Any rows and columns for letters
+        not in the standard alphabet are discarded.
 
-	Returns the matrix.
+        Returns the matrix.
 */
 /**********************************************************************/
 MATRIX_T *get_score_matrix(
-  char *score_filename,		/* name of score file */
-  BOOLEAN_T dna,		/* DNA alphabet if true; default: protein */
-  int dist,			/* PAM distance (ignored if filename != NULL) */
-  char *alpha			/* standard alphabet for final matrix */
+  char *score_filename,         /* name of score file */
+  ALPH_T* alph,                  /* alphabet */
+  int dist                      /* PAM distance (ignored if filename != NULL) */
 )
 {
-  char *alpha1;			/* initial alphabet */
-  MATRIX_T *tmp;		/* temporary score matrix */
-  MATRIX_T *matrix;		/* score matrix */
+  char *alpha1;                 /* initial alphabet */
+  MATRIX_T *tmp;                /* temporary score matrix */
+  MATRIX_T *matrix;             /* score matrix */
 
-  if (score_filename) {		/* read matrix from file */
+  assert(alph != NULL && (alph_is_builtin_dna(alph) || alph_is_builtin_protein(alph)));
+
+  if (score_filename) {         /* read matrix from file */
     tmp = read_score_matrix(score_filename, &alpha1);
-  } else {			/* generate PAM matrix */
-    tmp = gen_pam_matrix(dna, dist, TRUE);
-    alpha1 = dna ? pam_dna_alphabet : pam_prot_alphabet;
+    /* reorder the matrix to standard alphabet */
+    matrix = reorder_matrix(alpha1, alph, tmp);
+    free_matrix(tmp);
+    free(alpha1);
+  } else {                      /* generate PAM matrix */
+    matrix = gen_pam_matrix(alph, dist, true);
   }
-
-  /* reorder the matrix to standard alphabet */
-  matrix = reorder_matrix(alpha1, alpha, tmp);
-  free_matrix(tmp);
  
   return(matrix); 
 } /* get_score_matrix */
@@ -296,15 +300,15 @@ MATRIX_T *get_score_matrix(
 
 /**********************************************************************/
 /*
-	make_karlin_input
+        make_karlin_input
 
-	Prepare the input required for karlin() from a scoring matrix
-	and a letter frequency distribution.
+        Prepare the input required for karlin() from a scoring matrix
+        and a letter frequency distribution.
 */
 /**********************************************************************/
 KARLIN_INPUT_T *make_karlin_input(
-  MATRIX_T *matrix,			/* scoring matrix */
-  ARRAY_T *probs			/* letter freq distribution */
+  MATRIX_T *matrix,                     /* scoring matrix */
+  ARRAY_T *probs                        /* letter freq distribution */
 )
 {
   int i, j;
@@ -312,8 +316,8 @@ KARLIN_INPUT_T *make_karlin_input(
   long lowest, highest;
   ARRAY_T *score_probs; 
   int nscores;
-  int alen = get_num_rows(matrix);	/* size of alphabet */
-  KARLIN_INPUT_T *karlin_input;		/* data to return */
+  int alen = get_num_rows(matrix);      /* size of alphabet */
+  KARLIN_INPUT_T *karlin_input;         /* data to return */
 
   /*  find the highest and lowest scores in the scoring matrix */
   lowest = 1;
@@ -341,7 +345,7 @@ KARLIN_INPUT_T *make_karlin_input(
       double pi = get_array_item(i, probs);
       double pj = get_array_item(j, probs);
       double sp = get_array_item(s-lowest, score_probs); 
-      set_array_item(s-lowest, sp + pi*pj, score_probs);	/* cumulative prob. of score */
+      set_array_item(s-lowest, sp + pi*pj, score_probs);        /* cumulative prob. of score */
       escore += pi*pj*s;
       /*printf("i %d j %d s %d pi %f pj %f sp %f escore %f\n",i,j,s, pi, pj, sp, escore);*/
     }
@@ -359,21 +363,21 @@ KARLIN_INPUT_T *make_karlin_input(
 
 /**********************************************************************/
 /*
-	convert_score_to_target
+        convert_score_to_target
 
-	Convert a scoring matrix to a target matrix.
+        Convert a scoring matrix to a target matrix.
 */
 /**********************************************************************/
 MATRIX_T *convert_score_to_target(
-  MATRIX_T *score,			/* score matrix */
-  ARRAY_T *prob				/* letter frequencies */
+  MATRIX_T *score,                      /* score matrix */
+  ARRAY_T *prob                         /* letter frequencies */
 )
 {
   int i, j;
   KARLIN_INPUT_T *karlin_input;
   double lambda, K, H;
-  MATRIX_T *target;			/* target freq. matrix */
-  int alen = get_num_rows(score);	/* alphabet length */
+  MATRIX_T *target;                     /* target freq. matrix */
+  int alen = get_num_rows(score);       /* alphabet length */
 
   /* make input for karlin() */
   karlin_input = make_karlin_input(score, prob);
@@ -404,47 +408,46 @@ MATRIX_T *convert_score_to_target(
 
 /**********************************************************************/
 /*
-       	get_subst_target_matrix
+        get_subst_target_matrix
 
-	Get a substitution target frequency matrix.
+        Get a substitution target frequency matrix.
 */
 /**********************************************************************/
 MATRIX_T *get_subst_target_matrix(
-  char *score_filename,		/* name of score file */
-  BOOLEAN_T dna,		/* DNA alphabet if true; default: protein */
-  int dist,			/* PAM distance (ignored if score_filename != NULL) */
-  char *alpha,			/* standard alphabet for final matrix */
-  ARRAY_T *back			/* background frequencies of standard alphabet */
+  char *score_filename,         /* name of score file */
+  ALPH_T* alph,                  /* alphabet */
+  int dist,                     /* PAM distance (ignored if score_filename != NULL) */
+  ARRAY_T *back                 /* background frequencies of standard alphabet */
 )
 {
-  MATRIX_T *score;		/* score matrix */
-  MATRIX_T *target;		/* target frequency matrix */
+  MATRIX_T *score;              /* score matrix */
+  MATRIX_T *target;             /* target frequency matrix */
 
-  score = get_score_matrix(score_filename, dna, dist, alpha);
+  score = get_score_matrix(score_filename, alph, dist);
   target = convert_score_to_target(score, back);
 
   if (SUBST_MATRIX_DEBUG)
   {
-    int i, j, alength=strlen(alpha);
+    int i, j, alength=alph_size_core(alph);
     double sum;
 
       if (score_filename) {
-	printf("From file %s\n", score_filename);
+        printf("From file %s\n", score_filename);
       } else {
-	printf("Generated PAM %d\n", dist);
+        printf("Generated PAM %d\n", dist);
       }
       printf("%6c ", ' ');
       for (i=0; i<alength; i++) {
-	printf("%6c ", alpha[i]);
+        printf("%6c ", alph_char(alph, i));
       }
       printf("\n");
     sum = 0;
     for (i=0; i<alength; i++) {
-      printf("%6c ", alpha[i]);
+      printf("%6c ", alph_char(alph, i));
       for (j=0; j<alength; j++) {
-	double x = get_matrix_cell(i,j,score);
-	sum += x;
-	printf("%6.4f ", x);
+        double x = get_matrix_cell(i,j,score);
+        sum += x;
+        printf("%6.4f ", x);
       }
       printf("\n");
     }
@@ -465,22 +468,22 @@ MATRIX_T *get_subst_target_matrix(
  * the average of pseudocounts for letters matching the ambiguous ones.
  **************************************************************************/
 ARRAY_T *get_pseudocount_freqs(
-   ARRAY_T *	  f,		/* Foreground distribution. */
-   ARRAY_T *      b,		/* Background distribution. */
-   MATRIX_T *     target_freq	/* Target frequency matrix. */
+   ALPH_T* alph,
+   ARRAY_T *      f,            /* Foreground distribution. */
+   ARRAY_T *      b,            /* Background distribution. */
+   MATRIX_T *     target_freq   /* Target frequency matrix. */
 )
 {
   int i, j;
-  int alph_size = get_alph_size(ALL_SIZE);	// includes ambigs
-  int asize = get_alph_size(ALPH_SIZE);		// excludes ambigs
-  ARRAY_T *g = allocate_array(alph_size);
+  int asize = alph_size_core(alph);               // excludes ambigs
+  ARRAY_T *g = allocate_array(alph_size_full(alph));// includes ambigs
 
   /*
     Create pseudocount frequencies.
   */
-  for (i = 0; i < asize; i++) {				/* non-ambiguous freqs */
+  for (i = 0; i < asize; i++) {                         /* non-ambiguous freqs */
     double gi = 0;
-    for (j= 0; j < asize; j++) {			/* non-ambiguous freqs */
+    for (j= 0; j < asize; j++) {                        /* non-ambiguous freqs */
       double qij = get_matrix_cell(i, j, target_freq);
       double fj = get_array_item(j, f);
       double bj = get_array_item(j, b);
@@ -489,28 +492,28 @@ ARRAY_T *get_pseudocount_freqs(
     set_array_item(i, gi, g);
     if (SUBST_MATRIX_DEBUG) printf("%g %g, ", get_array_item(i, f), gi);
   } /* i */
-  fill_in_ambiguous_chars(FALSE, g);			/* takes the average pseudocount */
+  calc_ambigs(alph, false, g);                  /* takes the average pseudocount */
   if (SUBST_MATRIX_DEBUG) printf("\n");
 
-  return(g);						/* return the pseudocounts */
+  return(g);                                            /* return the pseudocounts */
 } /* get_pseudocount_freqs */
 
 #ifdef SO
 /***********************************************************************/
 /*
-	Testing routine.
-	
-	subst_matrix <0|1> <dist>
+        Testing routine.
+        
+        subst_matrix <0|1> <dist>
 
-	Specify 0 for protein, 1 for DNA.
-	Specify the integer PAM distance.
+        Specify 0 for protein, 1 for DNA.
+        Specify the integer PAM distance.
 */
 /***********************************************************************/
 VERBOSE_T verbosity = NORMAL_VERBOSE;
 main(int argc, char **argv) {
   int i, j, alength;
   int dist = 0;
-  BOOLEAN_T dna = FALSE; 
+  ALPH_T* alph;
   char *score_filename = NULL;
   char *alpha;
   MATRIX_T *matrix;
@@ -521,8 +524,10 @@ main(int argc, char **argv) {
   double sum;
   char usage[1000] = "";
 
+  alph = alph_protein();
+
   // Define the usage message.
-  strcat(usage, "USAGE: subst_matrix [options] <score file>\n");
+  strcat(usage, "Usage: subst_matrix [options] <score file>\n");
   strcat(usage, "\n");
   strcat(usage, "   Options:\n");
   strcat(usage, "     --dna\n");
@@ -552,7 +557,7 @@ main(int argc, char **argv) {
     // Get the option name (we only use long options).
     option_name = long_options[option_index].name;
     if (strcmp(option_name, "dna") == 0) {
-      dna = TRUE;
+      alph = DNA_ALPH;
     } else if (strcmp(option_name, "dist") == 0) {
       dist = atoi(optarg);
     } else {
@@ -569,29 +574,28 @@ main(int argc, char **argv) {
 
 
 
-  alpha = dna ? "ACGT" : "ACDEFGHIKLMNPQRSTVWY";
-  alength = strlen(alpha);
+  alength = alph_size_core(alph);
 
   /* background frequencies */
   probs = allocate_array(alength);
-  freqs = dna ? pam_dna_freq : pam_prot_freq;
-  fill_array(freqs, probs);			/* copy freqs into ARRAY_T */
+  freqs = alph == DNA_ALPH ? pam_dna_freq : pam_prot_freq;
+  fill_array(freqs, probs);                     /* copy freqs into ARRAY_T */
 
   if (dist > 1) {
     printf("From gen_pam_matrix:\n");
-    matrix = gen_pam_matrix(dna, dist, FALSE);
+    matrix = gen_pam_matrix(alph, dist, false);
     printf("%6c ", ' ');
     for (i=0; i<alength; i++) {
-      printf("%6c ", alpha[i]);
+      printf("%6c ", alph_char(alph, i));
     }
     printf("\n");
     sum = 0;
     for (i=0; i<alength; i++) {
-      printf("%6c ", alpha[i]);
+      printf("%6c ", alph_char(alph, i));
       for (j=0; j<alength; j++) {
-	double x = get_matrix_cell(i,j,matrix);
-	sum += x;
-	printf("%6.4f ", x);
+        double x = get_matrix_cell(i,j,matrix);
+        sum += x;
+        printf("%6.4f ", x);
       }
       printf("\n");
     }
@@ -599,6 +603,7 @@ main(int argc, char **argv) {
   }
 
   printf("From get_subst_target_matrix:\n");
-  matrix = get_subst_target_matrix(score_filename, dna, dist, alpha, probs);
+  matrix = get_subst_target_matrix(score_filename, alph, dist, probs);
+  alph_release(alph);
 } /* main */
 #endif
