@@ -1,4 +1,3 @@
-
 var alphabet = null;
 var sequences = null;
 var control_sequences = null;
@@ -38,6 +37,8 @@ function check() {
     if (alphs == null) alphs = sequences.get_alphabets();
     if (sequences.source.value == "file") {
       num_sequences = sequences.file_dbh.sequence_count;
+    } else if (sequences.source.value == "bedfile") {
+      num_sequences = sequences.bed_file_dbh.sequence_count;
     } else if (sequences.source.value == "text") {
       num_sequences = sequences.text_dbh.sequence_count;
     }
