@@ -921,9 +921,9 @@ HtmlMotifParser.prototype._html_data = function (data) {
   if (version == null || !/^\d+(?:\.\d+){0,2}$/.test(version)) {
     throw new MotifError("Invalid version string \"" + version + "\".");
   }
-  var program = data.program;
-  if (program == null || (program != "MEME" && program != "dreme" && program != "streme")) {
-    throw new MotifError("Unrecognized program, expected MEME, dreme or streme.");
+  var program = data.program.toUpperCase();
+  if (program == null || (program != "MEME" && program != "DREME" && program != "STREME")) {
+    throw new MotifError("Unrecognized program, expected MEME, STREME or DREME.");
   }
   this.rating = 3;
   var alphabetData = data.alphabet;
@@ -987,7 +987,7 @@ HtmlMotifParser.prototype._html_data = function (data) {
     }
     alphabet = new Alphabet(alphabetData);
     this.rating = 4;
-    if (program == "streme") {
+    if (program == "STREME") {
       norc = true;
     } else {
       if (typeof data.options != "object" || typeof data.options.revcomp != "boolean") {
@@ -1860,7 +1860,7 @@ XmlMotifParser.prototype._streme_motif = function (motif_node, alph_ids) {
   var nsites = motif_node.getAttribute("train_pos_count");
   if (nsites == null || !/^\d+$/.test(nsites)) throw new MotifError("STREME XML motif train_pos_count attribute is not a number.");
   nsites = +nsites;
-  if (nsites < 1) throw new MotifError("DREME XML motif train_pos_count attribute should be at least 1.");
+  if (nsites < 1) throw new MotifError("STREME XML motif train_pos_count attribute should be at least 1.");
   // read the evalue
   var pvalue = motif_node.getAttribute("test_pvalue");
   if (pvalue == null || !/^[+-]?\d+(?:\.\d+)?(?:[eE][+-]?\d+)?$/.test(pvalue)) {
@@ -1981,11 +1981,11 @@ XmlMotifParser.prototype.process_streme_doc = function (doc) {
   var i, version, alphabet, norc, background;
   // get the version
   var streme_ele = doc.querySelector("STREME");
-  if (streme_ele == null) throw new MotifError("STREME XML missing /streme .");
+  if (streme_ele == null) throw new MotifError("STREME XML missing /STREME.");
   version = streme_ele.getAttribute("version");
-  if (version == null) throw new MotifError("STREME XML missing /streme/@version .");
+  if (version == null) throw new MotifError("STREME XML missing /STREME/@version .");
   if (!/^\d+(?:\.\d+){0,2}$/.test(version)) {
-    throw new MotifError("STREME XML value /streme/@version does not match expected pattern; got \"" + version + "\".");
+    throw new MotifError("STREME XML value /STREME/@version does not match expected pattern; got \"" + version + "\".");
   }
   this.rating = 2;
   // get the alphabet
@@ -2000,9 +2000,9 @@ XmlMotifParser.prototype.process_streme_doc = function (doc) {
       alph_parser.parse_header(alphabet_ele.getAttribute("name"));
       for (i = 0; i < letter_elems.length; i++) {
         letter_id = letter_elems[i].getAttribute("id");
-        if (letter_id == null) throw new MotifError("STREME XML missing value /streme/model/alphabet/letter[" + i + "]/@id .");
+        if (letter_id == null) throw new MotifError("STREME XML missing value /STREME/model/alphabet/letter[" + i + "]/@id .");
         letter_symbol = letter_elems[i].getAttribute("symbol");
-        if (letter_symbol == null) throw new MotifError("STREME XML missing value /streme/model/alphabet/letter[" + i + "]/@symbol .");
+        if (letter_symbol == null) throw new MotifError("STREME XML missing value /STREME/model/alphabet/letter[" + i + "]/@symbol .");
         sym2id[letter_symbol] = letter_id;
         letter_aliases = letter_elems[i].getAttribute("aliases");
         letter_name = letter_elems[i].getAttribute("name");
@@ -2032,10 +2032,35 @@ XmlMotifParser.prototype.process_streme_doc = function (doc) {
     alphabet_ids = ["A", "C", "G", "T"];
   }
   this.rating = 3;
+  // make a list of the letter_ids in order of the alphabet
+  var symbol_map = {};
+  for (i = 0; i < letter_elems.length; i++) {
+    letter_symbol = letter_elems[i].getAttribute("symbol");
+    if (letter_symbol == null) throw new MotifError("MEME XML missing value /MEME/training_set/alphabet/letter[" + i + "]/@symbol .");
+    letter_id = letter_elems[i].getAttribute("id");
+    if (letter_id == null) throw new MotifError("MEME XML missing value /MEME/training_set/alphabet/letter[" + i + "]/@id .");
+    symbol_map[letter_symbol] = letter_id;
+  }
+  var symbol_ids = [];
+  for (i = 0; i < alphabet.get_size_core(); i++) {
+    var id = symbol_map[alphabet.get_symbol(i)];
+    if (id == null) throw new MotifError("MEME XML missing value /MEME/training_set/alphabet/letter[@symbol='" + alphabet.get_symbol(i) + "']/@id .");
+    symbol_ids.push(id);
+  }
   this.rating = 4;
-  // get the background
-  var background_ele = doc.querySelector('STREME > model > background');
-  background = this._streme_probability_array(background_ele, alphabet_ids);
+  // try to get the background_frequencies (post 5.3.0)
+  var background_ele = doc.querySelector("STREME > model > background_frequencies > alphabet_array");
+  if (background_ele != null) {
+    background = this._meme_alphabet_array(background_ele, symbol_ids, true);
+  } else {
+    // try to get the background (removed after 5.3.0)
+    var background_ele = doc.querySelector('STREME > model > background');
+    if (background_ele != null) {
+      background = this._streme_probability_array(background_ele, alphabet_ids);
+    } else {
+      throw new MotifError("STREME XML missing /STREME/model/background_frequencies/alphabet_array .");
+    }
+  }
   this.rating = 5;
   // send the meta data
   if (typeof this.handler.meta == "function") {
