@@ -167,6 +167,11 @@ function create_job_detail_sequence(item, inputs, even) {
       file.appendChild(document.createTextNode(input["safe-file"]));
       value.appendChild(file);
     }
+    if (input["genome"] != null) {
+      text = " referencing the sequence database ";
+      text += input["genome"];
+      value.appendChild(document.createTextNode(text));
+    }
   } else if (input["source"] == "db") {
     value.appendChild(document.createTextNode(" from the database "));
     file = document.createElement("span");
@@ -264,21 +269,6 @@ function create_job_detail_gomo(item, inputs, even) {
   return create_row(even, item["name"], value);
 }
 
-function create_job_detail_loci(item, inputs, even) {
-  "use strict";
-  var input, fileOrig, fileSafe, text, value;
-  input = inputs["loci"];
-  text = input["safe-file"];
-  fileOrig = document.createElement("span");
-  fileOrig.className = "file";
-  fileOrig.appendChild(document.createTextNode(input["orig-file"]));
-  fileSafe = document.createElement("span");
-  fileSafe.className = "file";
-  fileSafe.appendChild(document.createTextNode(input["safe-file"]));
-  value = replace_tokens(text, "!!ORIG-NAME!!", fileOrig, "!!SAFE-NAME!!", fileSafe);
-  return create_row(even, item["name"], value);
-}
-
 function create_job_detail_background(item, inputs, even) {
   "use strict";
   var input, source, order, file, text, fileOrig, fileSafe, value;
@@ -316,7 +306,7 @@ function create_job_detail_file(item, inputs, even) {
     return null;
   }
   input = inputs[item["key"]];
-  text = (input["safe-file"] === input["orig-file"] ? item["normal"] : item["rename"]);
+  text = input["safe-file"];
   fileOrig = document.createElement("span");
   fileOrig.className = "file";
   fileOrig.appendChild(document.createTextNode(input["orig-file"]));
@@ -522,8 +512,6 @@ function create_job_row(item, inputs, even) {
   type = item["type"];
   if (type == "sequences") {
     return create_job_detail_sequence(item, inputs, even);
-  } else if (type == "loci") {
-    return create_job_detail_loci(item, inputs, even);
   } else if (type == "motifs") {
     return create_job_detail_motif(item, inputs, even);
   } else if (type == "gomo") {
