@@ -97,6 +97,10 @@ public class Ame extends SubmitJob<Ame.Data> {
       }
       addArgs(args, "-control_type", control_type);
       if (control != null) {
+        if (control instanceof LociDataSource) {
+          addArgs(args, "-bedfile-neg");
+          addArgs(args, "-genome-neg", ((LociDataSource)control).getGenomeFileName());
+        }
         addArgs(args, "-control", control.getName());
       }
       addArgs(args, "-method", method);
@@ -117,6 +121,10 @@ public class Ame extends SubmitJob<Ame.Data> {
       } else if (background.getSource() == Background.Source.MEME) {
         addArgs(args, "-bgfile", "--motif--"); // Use background in motif file
       }
+      if (sequences instanceof LociDataSource) {
+        addArgs(args, "-bedfile-pos");
+        addArgs(args, "-genome-pos", ((LociDataSource)sequences).getGenomeFileName());
+      }
       addArgs(args, sequences.getName());
       if (motifs instanceof MotifDataSource) {
         addArgs(args, ((MotifDataSource) motifs).getName());
