@@ -401,106 +401,6 @@ public class Tomtom extends SubmitJob<Tomtom.Data> {
   protected void submitOpalJob(UUID uuid, Data data, HttpServletResponse response)
       throws ServletException, IOException, QuotaException {
     String drmaaQueueShort = msp.getDrmaaQueueShort();
-    // Use Opal if not an immediate run job or if there is a queue for short jobs.
-    if (!data.immediateRun || drmaaQueueShort.length() != 0) {
-      super.submitOpalJob(uuid, data, response);
-    } else {
-      // test the job quota
-      Tracker tracker = jobTable.addJob(uuid.toString());
-      // skip sending to opal, just run tomtom_webservice and redirect
-      File resultDir = null;
-      try {
-        String jobName = "appTOMTOM_" + msp.getVersion() + "_" + System.currentTimeMillis() + rand.nextInt();
-        resultDir = createResultDir(jobName);
-        // store the data files needed by the webservice
-        for (DataSource source : data.files()) storeSource(resultDir, source);
-        if (data.description != null) storeString(resultDir, "description", data.description());
-        storeString(resultDir, "uuid", uuid.toString());
-        storeString(resultDir, "submit_time_file", loggableDate(new Date()));
-
-        // run the script
-        List<String> tomtom_args = data.args();
-        tomtom_args.add(0, new File(msp.getLibExecDir(), "tomtom_webservice").toString());
-        ProcessBuilder pb = new ProcessBuilder();
-        pb.directory(resultDir); // set the running directory
-        pb.command(tomtom_args); // set the command
-        Process tomtom;
-        // Acquire the semaphore throtlling immediate mode Tomtom jobs
-        boolean haveSem = false;
-        try {
-          haveSem = tomtomSem.tryAcquire(100, TimeUnit.SECONDS);
-        } catch (InterruptedException e) {
-          throw new ServletException("Service interrupted while trying to acquire tomtom semaphore.  " +
-           "Please try Tomtom again later or un-check the 'Run immediately' box.");
-        }
-        if (haveSem) {
-          // Start Tomtom.
-          tomtom = pb.start();
-        } else {
-          throw new ServletException("Too many Tomtom jobs are running currently.  " +
-           "Please try Tomtom again later or un-check the 'Run immediately' box.");
-        }
-
-        // setup a thread to consume any messages and print them to console
-        StreamCopier.toSystemOut(tomtom.getInputStream(), true);
-        StreamCopier.toSystemErr(tomtom.getErrorStream(), true);
-
-        // setup a thread to wait for the process so we can wait for it (until a timeout)
-        Waiter waiter = new Waiter(tomtom);
-        waiter.start();
-
-        Integer exitValue = null;
-        boolean stillAlive = true;
-        try {
-          waiter.join();
-          stillAlive = waiter.isAlive();
-          try {
-            // Only assign an exit value if the process has stopped running
-            if (!stillAlive) {
-              exitValue = tomtom.exitValue();
-            }
-          } catch (IllegalThreadStateException e) {
-            throw new ServletException("Tomtom did not complete within time limit for unqueued jobs.");
-          }
-        } catch (InterruptedException e) {
-          throw new ServletException(e);
-        } finally {
-          // Destroy doesn't work on Ubuntu
-          tomtom.destroy();
-          // Release the semaphore file if we have it.
-          if (haveSem) {
-            tomtomSem.release();
-            //System.out.println("released lock");
-          }
-        }
-
-        // now check that it worked.
-        if (exitValue == null) {
-          throw new ServletException("Tomtom did not complete within time limit for unqueued jobs.");
-        } else if (exitValue != 0) {
-          // TLB; Useful error messages will be in the "Error Messages" file
-          // so don't throw an exception!
-          // throw new ServletException("Tomtom returned non-zero exit status.");
-        }
-        // check that a index.html output was created
-        File result = new File(resultDir, "index.html");
-        if (!result.isFile()) {
-          throw new ServletException("Tomtom did not create an index.html file.");
-        }
-        URL siteURL = new URL(msp.getSiteURL());
-        String outputURLString = siteURL.getProtocol() +"://" + siteURL.getHost();
-        int port = siteURL.getPort();
-        if (port > 0) {
-          // The site URL contains an explicit port value
-          // so repeat that here.
-          outputURLString = outputURLString + ":" + Integer.toString(port);
-        }
-        outputURLString = outputURLString + "/opal-jobs/" + jobName;
-        response.sendRedirect(outputURLString );
-      } finally {
-        tracker.setFinished();
-      }
-    }
+    super.submitOpalJob(uuid, data, response);
   }
-
 }
