@@ -695,12 +695,27 @@ public class WebUtils {
       boolean png, Double width, Double height)
       throws IOException, ServletException {
     File motifFile = null, imgFile = null;
+    String outName = "";
     try {
       // create input and output temporary files
       motifFile = File.createTempFile("motifs_", ".meme");
       motifFile.deleteOnExit();
       imgFile = File.createTempFile("logo_", (png ? ".png" : ".eps"));
       imgFile.deleteOnExit();
+      // create name of output file
+      if (id1.matches("^[+-]?\\d+$")) {
+        // id1 is a motif index
+        outName = "logo";
+      } else {		
+        // id1 is a motif id
+        outName = id1;
+        if (rc1) outName += "_rc";
+        if (id2 != null) {
+          outName += "_" + id2;
+          if (rc2) outName += "_rc";
+        }
+     }
+     outName += (png ? ".png" : ".eps");
       // save the motif text to a temporary file
       BufferedWriter writer = null;
       try {
@@ -764,8 +779,7 @@ public class WebUtils {
       if (ceqlogo.exitValue() != 0) {
         throw new ServletException("Ceqlogo returned non-zero exit status");
       }
-      outputFile(resp, imgFile, "logo." + (png ? "png" : "eps"),
-          (png ? "image/png" : "application/postscript"));
+      outputFile(resp, imgFile, outName, (png ? "image/png" : "application/postscript"));
     } finally {
       if (motifFile != null) {
         if (!motifFile.delete()) logger.log(Level.WARNING, "Failed to delete " + motifFile);
