@@ -26,7 +26,8 @@ import java.util.logging.SimpleFormatter;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import static au.edu.uq.imb.memesuite.db.SQL.*;
+//import static au.edu.uq.imb.memesuite.db.SQL.*;
+import au.edu.uq.imb.memesuite.db.SQL;
 import static au.edu.uq.imb.memesuite.util.MultiSourceStatus.*;
 
 /**
@@ -66,15 +67,15 @@ public class UpdateSequenceDB {
       connection = ds.getConnection();
       Statement statement = connection.createStatement();
       logger.log(Level.INFO, "Creating category table");
-      statement.executeUpdate(CREATE_TBL_CATEGORY);
+      statement.executeUpdate(SQL.CREATE_TBL_CATEGORY);
       logger.log(Level.INFO, "Creating listing table");
-      statement.executeUpdate(CREATE_TBL_LISTING);
+      statement.executeUpdate(SQL.CREATE_TBL_LISTING);
       logger.log(Level.INFO, "Creating sequence table");
-      statement.executeUpdate(CREATE_TBL_SEQUENCE_FILE);
+      statement.executeUpdate(SQL.CREATE_TBL_SEQUENCE_FILE);
       logger.log(Level.INFO, "Creating priors table");
-      statement.executeUpdate(CREATE_TBL_PRIOR_FILE);
+      statement.executeUpdate(SQL.CREATE_TBL_PRIOR_FILE);
       logger.log(Level.INFO, "Declaring the schema version");
-      statement.executeUpdate(DECLARE_VERSION);
+      statement.executeUpdate(SQL.DECLARE_VERSION);
       statement.close();
       connection.close();
       connection = null;
@@ -157,7 +158,7 @@ public class UpdateSequenceDB {
   private static int queryDbVersion(Connection connection) throws SQLException {
     int version = 0;
     Statement statement = connection.createStatement();
-    ResultSet resultSet = statement.executeQuery(QUERY_VERSION);
+    ResultSet resultSet = statement.executeQuery(SQL.QUERY_VERSION);
     if (resultSet.next()) {
       version = resultSet.getInt(1);
     }
@@ -239,8 +240,7 @@ public class UpdateSequenceDB {
     TableSchema[] schemas = new TableSchema[] {
         new TableSchema("tblCategory").addCol("id", "INTEGER").addCol("name", "TEXT"),
         new TableSchema("tblListing").addCol("id", "INTEGER").
-            addCol("categoryId", "INTEGER").addCol("name", "TEXT").
-            addCol("description", "TEXT"),
+            addCol("categoryId", "INTEGER").addCol("name", "TEXT"),
         new TableSchema("tblSequenceFile").addCol("id", "INTEGER").
             addCol("retriever", "INTEGER").addCol("listingId", "INTEGER").
             addCol("alphabet", "INTEGER").addCol("edition", "INTEGER").
@@ -262,7 +262,7 @@ public class UpdateSequenceDB {
       if (version == 0) {
         logger.log(Level.SEVERE, "The database schema version is missing.");
         ok = false;
-      } else if (version < SCHEMA_VERSION) {
+      } else if (version < SQL.SCHEMA_VERSION) {
         switch (version) {
           case 1:
             upgradeV1ToV2(ds);
@@ -274,7 +274,7 @@ public class UpdateSequenceDB {
             logger.log(Level.SEVERE, "The database schema is older than handled by this program.");
             ok = false;
         }
-      } else if (version > SCHEMA_VERSION) {
+      } else if (version > SQL.SCHEMA_VERSION) {
         logger.log(Level.SEVERE, "The database schema is newer than handled by this program.");
         ok = false;
       }
@@ -452,7 +452,7 @@ public class UpdateSequenceDB {
    * @param dbDir the directory containing the fasta databases.
    * @param status the status.
    */
-  public static void purge(SQLiteDataSource ds, File binDir, File dbDir, boolean warnOnly, MultiSourceStatus status) throws InterruptedException {
+  public static void purge(SQLiteDataSource ds, File binDir, File dbDir, boolean warnOnly, String fileSeqPattern, MultiSourceStatus status) throws InterruptedException {
     Connection conn = null;
     int regeneratedBgs = 0;
     int deletedFiles = 0;
@@ -465,7 +465,9 @@ public class UpdateSequenceDB {
       conn = ds.getConnection();
       conn.setAutoCommit(false);
       // select fileSeq, fileBg from tblSequenceFile
-      PreparedStatement pstmt = conn.prepareStatement(SQL.SELECT_ALL_SEQUENCE_FILES);
+      //PreparedStatement pstmt = conn.prepareStatement(SQL.SELECT_ALL_SEQUENCE_FILES);
+      PreparedStatement pstmt = conn.prepareStatement(SQL.SELECT_SEQUENCE_FILES_BY_FILESEQ);
+      pstmt.setString(1, fileSeqPattern);
       PreparedStatement pstmtDelFile = conn.prepareStatement(SQL.DELETE_SEQUENCE_FILE);
       ResultSet rset = pstmt.executeQuery();
       while (rset.next()) {
@@ -620,13 +622,45 @@ public class UpdateSequenceDB {
       // initialize and launch all the downloader threads
       List<Future<?>> downloaderTasks = new ArrayList<Future<?>>();
       for (String className : updaters) {
+
         try {
-          // use some reflection to find the constructor
+          // Special handling for Ensembl (and Genbank and UCSC) divisions; 
+          // use the EnsemblUpdater (or GenbankUpdater and UCSC) 
+	  // with the division specified in its call.
+          String division = "unknown_division";
+          Pattern ensemblMethod = Pattern.compile("^ensembl.*[abinitio]{0,1}.*$");
+          Matcher m = ensemblMethod.matcher(className);
+          if (m.matches()) { 
+            division = className;
+            className = "au.edu.uq.imb.memesuite.updatedb.EnsemblUpdater";
+          } else {
+            Pattern genbankMethod = Pattern.compile("^genbank.*$");
+            m = genbankMethod.matcher(className);
+	    if (m.matches()) { 
+	      division = className;
+	      className = "au.edu.uq.imb.memesuite.updatedb.GenbankUpdater";
+            } else {
+              Pattern ucscMethod = Pattern.compile("^ucsc.*$");
+              m = ucscMethod.matcher(className);
+	      if (m.matches()) { 
+		division = className;
+		className = "au.edu.uq.imb.memesuite.updatedb.UCSCGenomeUpdater";
+	      } else {
+		Pattern rsatMethod = Pattern.compile("^rsat.*$");
+		m = rsatMethod.matcher(className);
+		if (m.matches()) { 
+		  division = className;
+		  className = "au.edu.uq.imb.memesuite.updatedb.RsatUpdater";
+                }
+	      }
+	    }
+          }
+          // Create the constructor for the updater.
           Class<?> updaterClass = Class.forName(className);
           Constructor<?> updaterConstructor = updaterClass.getConstructor(
-              SQLiteDataSource.class, ReentrantReadWriteLock.class, File.class,
+              String.class, SQLiteDataSource.class, ReentrantReadWriteLock.class, File.class,
               File.class, ExecutorService.class, MultiSourceStatus.class);
-          Object updater = updaterConstructor.newInstance(ds, lock, binDir,
+          Object updater = updaterConstructor.newInstance(division, ds, lock, binDir,
               dbDir, sequenceWorker, status);
           // launch the task
           downloaderTasks.add(downloadWorkers.submit((Callable<?>)updater));
@@ -1091,7 +1125,6 @@ public class UpdateSequenceDB {
     return binDir;
   }
 
-
   /**
    * Run the program.
    * @param args the program arguments.
@@ -1117,16 +1150,12 @@ public class UpdateSequenceDB {
       // add the class to the list of downloaders
       String qualifiedClass = setup.getProperty("dl." + id + ".class");
       updaters.add(qualifiedClass);
+
       // add an option to enable the downloader
       Switch aSwitch = new Switch(qualifiedClass).setLongFlag(id);
       String description = setup.getProperty("dl." + id + ".description", id);
       if (description != null) aSwitch.setHelp("Enable downloading " + description + ".");
       parameters.add(aSwitch);
-      // add an option to disable the downloader
-      aSwitch = new Switch("no_" + qualifiedClass).setLongFlag("no_" + id);
-      description = setup.getProperty("dl." + id + ".description", id);
-      if (description != null) aSwitch.setHelp("Disable downloading " + description + ".");
-      parameters.add(aSwitch);
     }
 
     // define some special parsers for files
@@ -1184,6 +1213,11 @@ public class UpdateSequenceDB {
           .setLongFlag("verbosity").setShortFlag('v').setDefault("3")
           .setHelp("Specify the logging level [1-8].")
     );
+    parameters.add(
+      new Switch("purge_only")
+          .setLongFlag("purge_only")
+          .setHelp("Delete sequence databases for which the files have been lost and exit.")
+    );
     parameters.add(
       new UnflaggedOption("db_dir").setStringParser(DIR_PARSER).setRequired(true)
           .setHelp("Specify the directory containing the databases.")
@@ -1191,9 +1225,11 @@ public class UpdateSequenceDB {
 
     SimpleJSAP jsap = new SimpleJSAP(
       "update-sequence-db",
-      "Update sequence databases; optionally remove old databases.",
+      "  Updates sequence databases; optionally removes old databases.",
       parameters.toArray(new Parameter[parameters.size()])
     );
+
+    // Parse the arguments.
     JSAPResult config = jsap.parse(args);
     if (jsap.messagePrinted()) {
       System.exit(1);
@@ -1207,17 +1243,22 @@ public class UpdateSequenceDB {
       break;
     }
 
+    // add any additional updaters
+    int n_specified = updaters.size();
+    updaters.addAll(Arrays.asList(config.getStringArray("updater")));
+    if (updaters.size() > n_specified) { specified_only = true; }
+
     // Remove all but specified updaters if one or more are given.
     iter = updaters.iterator();
     if (specified_only) {
       while (iter.hasNext()) if (! config.getBoolean(iter.next())) iter.remove();
-    } else {
-      // remove any disabled updaters
-      while (iter.hasNext()) if (config.getBoolean("no_" + iter.next())) iter.remove();
     }
 
-    // add any additional updaters
-    updaters.addAll(Arrays.asList(config.getStringArray("updater")));
+    // Error if no updaters given.
+    if (! specified_only && ! config.getBoolean("purge_only", false)) {
+      System.err.println("Error: You must specify at least one database to update.");
+      System.exit(1);
+    }
 
     // determine bin dir
     File binDir;
@@ -1263,7 +1304,9 @@ public class UpdateSequenceDB {
       // are we marking obsolete databases?
       String[] obsoleteGlobs = config.getStringArray("obsolete");
       File priorsFile = config.getFile("load_priors");
-      if (obsoleteGlobs != null && obsoleteGlobs.length > 0) {
+      if (config.getBoolean("purge_only", false)) {
+	purge(ds, binDir, dbDir, false, "%", status);
+      } else if (obsoleteGlobs != null && obsoleteGlobs.length > 0) {
         Progress progress = new Progress(status, "Marking files obsolete");
         for (String obsoleteGlob : obsoleteGlobs) {
           progress.setTask("GLOB " + obsoleteGlob);
@@ -1273,11 +1316,21 @@ public class UpdateSequenceDB {
       } else if (priorsFile != null) {
         loadPriors(ds, dbDir, priorsFile, status);
       } else {
-        // purge missing databases
-        purge(ds, binDir, dbDir, config.getBoolean("retain_missing", false), status);
+        // purge missing databases (of selected types)
+        if (updaters.size() > 0) {
+          Pattern actualUpdater = Pattern.compile("^au\\.edu.*$");
+          for (String className : updaters) {
+            Matcher m = actualUpdater.matcher(className);
+            String fileSeqPattern = "%";
+            if (! m.matches()) {
+              fileSeqPattern = className + "%";
+            }
+            purge(ds, binDir, dbDir, config.getBoolean("retain_missing", false), fileSeqPattern, status);
+          }
+        }
         // remove obsolete databases
         if (config.getBoolean("remove_obsolete")) removeObsolete(ds, dbDir);
-        // run the selected updaters
+        // ** run the selected updaters **
         update(ds, binDir, dbDir, updaters, status);
         // create the optional backwards compatibility views
         if (config.getBoolean("csv_dir")) {
